@@ -1,11 +1,14 @@
 #!/bin/sh
 
-LTV="libtoolize (GNU libtool) 1.3.2"
+export CFLAGS
+export LDFLAGS
+
+LTV="libtoolize (GNU libtool) 1.3.3"
 ACV="Autoconf version 2.13"
 AMV="automake (GNU automake) 1.4"
 USAGE="
 You need to install:
-	libtool-1.3.2
+	libtool-1.3.3
 	autoconf-2.13
 	automake-1.4
 "
@@ -26,7 +29,7 @@ if [ "$1" = "--noconfigure" ]; then
 fi
 
 if [ X"$@" = X  -a "X`uname -s`" = "XLinux" ]; then
-    ./configure --prefix=/usr
+    ./configure --prefix=/usr --enable-shared "$@"
 else
     ./configure "$@"
 fi
