@@ -1,7 +1,6 @@
 #include "system.h"
 
-#include "build/rpmbuild.h"
-#include "popt/popt.h"
+#include "rpmbuild.h"
 #include "build.h"
 #include "install.h"
 
@@ -45,6 +44,44 @@ static int checkSpec(Header h)
     return rc;
 }
 
+/*
+ * Kurwa, durni ameryka?ce sobe zawsze my?l?, ?e ca?y ?wiat mwi po
+ * angielsku...
+ */
+/* XXX this is still a dumb test but at least it's i18n aware */
+static int isSpecFile(const char *specfile)
+{
+    char buf[256];
+    const char * s;
+    FD_t fd;
+    int count;
+    int checking;
+
+    if (fdFileno((fd = fdOpen(specfile, O_RDONLY, 0))) < 0) {
+	fprintf(stderr, _("Unable to open spec file: %s\n"), specfile);
+	return 0;
+    }
+    count = fdRead(fd, buf, sizeof(buf));
+    fdClose(fd);
+
+    checking = 1;
+    for (s = buf; count--; s++) {
+	switch (*s) {
+	case '\r':
+	case '\n':
+	    checking = 1;
+	    break;
+	case ':':
+	    checking = 0;
+	    break;
+	default:
+	    if (checking && !(isprint(*s) || isspace(*s))) return 0;
+	    break;
+	}
+    }
+    return 1;
+}
+
 static int buildForTarget(const char *arg, struct rpmBuildArguments *ba,
 	const char *passPhrase, int fromTarball, char *cookie,
 	int force, int nodeps)
@@ -52,13 +89,10 @@ static int buildForTarget(const char *arg, struct rpmBuildArguments *ba,
     int buildAmount = ba->buildAmount;
     const char *buildRoot = ba->buildRootOverride;
     int test = ba->noBuild;
-
     FILE *f;
     const char * specfile;
     int res = 0;
     struct stat statbuf;
-    char * s;
-    int count, fd;
     char buf[BUFSIZ];
     Spec spec = NULL;
 
@@ -167,23 +201,14 @@ static int buildForTarget(const char *arg, struct rpmBuildArguments *ba,
 	fprintf(stderr, _("File is not a regular file: %s\n"), specfile);
 	return 1;
     }
-    
-    if ((fd = open(specfile, O_RDONLY)) < 0) {
-	fprintf(stderr, _("Unable to open spec file: %s\n"), specfile);
+
+    /* Try to verify that the file is actually a specfile */
+    if (!isSpecFile(specfile)) {
+	fprintf(stderr, _("File %s does not appear to be a specfile.\n"),
+		specfile);
 	return 1;
     }
-    count = read(fd, buf, sizeof(buf) < 128 ? sizeof(buf) : 128);
-    close(fd);
-    s = buf;
-    while(count--) {
-	if (! (isprint(*s) || isspace(*s) || (*s == 0x1b))) {
-	    fprintf(stderr, _("File contains non-printable characters(%c): %s\n"), *s,
-		    specfile);
-	    return 1;
-	}
-	s++;
-    }
-
+    
     /* Parse the spec file */
 #define	_anyarch(_f)	\
 (((_f)&(RPMBUILD_PREP|RPMBUILD_BUILD|RPMBUILD_INSTALL|RPMBUILD_PACKAGEBINARY)) == 0)
@@ -271,41 +296,44 @@ static int useCatalog = 0;
 
 static void buildArgCallback(poptContext con, enum poptCallbackReason reason,
                              const struct poptOption * opt, const char * arg,
-                             struct rpmBuildArguments * data)
+                             const void * data)
 {
+    struct rpmBuildArguments * rba = (struct rpmBuildArguments *) data;
+
     switch (opt->val) {
-    case POPT_USECATALOG: data->useCatalog = 1; break;
-    case POPT_NOBUILD: data->noBuild = 1; break;
-    case POPT_NOLANG: data->noLang = 1; break;
-    case POPT_SHORTCIRCUIT: data->shortCircuit = 1; break;
-    case POPT_RMSOURCE: data->buildAmount |= RPMBUILD_RMSOURCE; break;
-    case POPT_RMSPEC: data->buildAmount |= RPMBUILD_RMSPEC; break;
-    case POPT_RMBUILD: data->buildAmount |= RPMBUILD_RMBUILD; break;
+    case POPT_USECATALOG: rba->useCatalog = 1; break;
+    case POPT_NOBUILD: rba->noBuild = 1; break;
+    case POPT_NOLANG: rba->noLang = 1; break;
+    case POPT_SHORTCIRCUIT: rba->shortCircuit = 1; break;
+    case POPT_RMSOURCE: rba->buildAmount |= RPMBUILD_RMSOURCE; break;
+    case POPT_RMSPEC: rba->buildAmount |= RPMBUILD_RMSPEC; break;
+    case POPT_RMBUILD: rba->buildAmount |= RPMBUILD_RMBUILD; break;
     case POPT_BUILDROOT:
-	if (data->buildRootOverride) {
+	if (rba->buildRootOverride) {
 	    fprintf(stderr, _("buildroot already specified"));
 	    exit(EXIT_FAILURE);
+	    /*@notreached@*/
 	}
-	data->buildRootOverride = strdup(arg);
+	rba->buildRootOverride = xstrdup(arg);
 	break;
     case POPT_BUILDARCH:
 	fprintf(stderr, _("--buildarch has been obsoleted.  Use the --target option instead.\n")); 
 	exit(EXIT_FAILURE);
-	break;
+	/*@notreached@*/ break;
     case POPT_BUILDOS:
 	fprintf(stderr, _("--buildos has been obsoleted.  Use the --target option instead.\n")); 
 	exit(EXIT_FAILURE);
-	break;
+	/*@notreached@*/ break;
     case POPT_TARGETPLATFORM:
-	if (data->targets) {
-	    int len = strlen(data->targets) + strlen(arg) + 2;
-	    data->targets = realloc(data->targets, len);
-	    strcat(data->targets, ",");
+	if (rba->targets) {
+	    int len = strlen(rba->targets) + strlen(arg) + 2;
+	    rba->targets = xrealloc(rba->targets, len);
+	    strcat(rba->targets, ",");
 	} else {
-	    data->targets = malloc(strlen(arg) + 1);
-	    data->targets[0] = '\0';
+	    rba->targets = xmalloc(strlen(arg) + 1);
+	    rba->targets[0] = '\0';
 	}
-	strcat(data->targets, arg);
+	strcat(rba->targets, arg);
 	break;
     }
 }
