@@ -12,7 +12,7 @@
 
 struct hardLink {
     struct hardLink * next;
-    char ** files;		/* there are nlink of these, used by install */
+    const char ** files;	/* nlink of these, used by install */
     int * fileMaps;		/* used by build */
     dev_t dev;
     ino_t inode;
@@ -50,10 +50,11 @@ struct cpioHeader {
     time_t mtime;
     long size;
     dev_t dev, rdev;
-    char * path;
+    /*@owned@*/char * path;
 };
 
-static inline off_t saferead(CFD_t *cfd, void * vbuf, size_t amount) {
+static inline off_t saferead(CFD_t *cfd, /*@out@*/void * vbuf, size_t amount)
+{
     off_t rc = 0;
     char * buf = vbuf;
 
@@ -81,6 +82,11 @@ static inline off_t saferead(CFD_t *cfd, void * vbuf, size_t amount) {
 	case cpioIoTypeGzFd:
 		nb = gzdRead(cfd->cpioGzFd, buf, amount);
 		break;
+#if ENABLE_BZIP2_PAYLOAD
+	case cpioIoTypeBzFd:
+		nb = bzdRead(cfd->cpioBzFd, buf, amount);
+		break;
+#endif
 	}
 	if (nb <= 0)
 		return nb;
@@ -93,14 +99,16 @@ static inline off_t saferead(CFD_t *cfd, void * vbuf, size_t amount) {
     return rc;
 }
 
-static inline off_t ourread(CFD_t * cfd, void * buf, size_t size) {
+static inline off_t ourread(CFD_t * cfd, /*@out@*/void * buf, size_t size)
+{
     off_t i = saferead(cfd, buf, size);
     if (i > 0)
 	cfd->cpioPos += i;
     return i;
 }
 
-static inline void padinfd(CFD_t * cfd, int modulo) {
+static inline void padinfd(CFD_t * cfd, int modulo)
+{
     int buf[10];
     int amount;
     
@@ -108,9 +116,10 @@ static inline void padinfd(CFD_t * cfd, int modulo) {
     (void)ourread(cfd, buf, amount);
 }
 
-static inline off_t safewrite(CFD_t *cfd, void * vbuf, size_t amount) {
+static inline off_t safewrite(CFD_t *cfd, const void * vbuf, size_t amount)
+{
     off_t rc = 0;
-    char * buf = vbuf;
+    const char * buf = vbuf;
 
     while (amount > 0) {
 	size_t nb;
@@ -136,6 +145,11 @@ static inline off_t safewrite(CFD_t *cfd, void * vbuf, size_t amount) {
 	case cpioIoTypeGzFd:
 		nb = gzdWrite(cfd->cpioGzFd, buf, amount);
 		break;
+#if ENABLE_BZIP2_PAYLOAD
+	case cpioIoTypeBzFd:
+		nb = bzdWrite(cfd->cpioBzFd, buf, amount);
+		break;
+#endif
 	}
 	if (nb <= 0)
 		return nb;
@@ -149,7 +163,8 @@ static inline off_t safewrite(CFD_t *cfd, void * vbuf, size_t amount) {
     return rc; 
 }
 
-static inline int padoutfd(CFD_t * cfd, size_t * where, int modulo) {
+static inline int padoutfd(CFD_t * cfd, size_t * where, int modulo)
+{
     static int buf[10] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
     int amount;
     
@@ -161,7 +176,8 @@ static inline int padoutfd(CFD_t * cfd, size_t * where, int modulo) {
     return 0;
 }
 
-static int strntoul(const char * str, char ** endptr, int base, int num) {
+static int strntoul(const char *str, /*@out@*/char **endptr, int base, int num)
+{
     char * buf, * end;
     unsigned long ret;
 
@@ -173,7 +189,7 @@ static int strntoul(const char * str, char ** endptr, int base, int num) {
     if (*end)
 	*endptr = ((char *)str) + (end - buf);	/* XXX discards const */
     else
-	*endptr = "";
+	*endptr = ((char *)str) + strlen(str);
 
     return strtoul(buf, endptr, base);
 }
@@ -185,7 +201,8 @@ static int strntoul(const char * str, char ** endptr, int base, int num) {
 	sprintf(space, "%8.8lx", (unsigned long) (val)); \
 	memcpy(phys, space, 8);
 
-static int getNextHeader(CFD_t * cfd, struct cpioHeader * chPtr) {
+static int getNextHeader(CFD_t * cfd, struct cpioHeader * chPtr)
+{
     struct cpioCrcPhysicalHeader physHeader;
     int nameSize;
     char * end;
@@ -194,8 +211,8 @@ static int getNextHeader(CFD_t * cfd, struct cpioHeader * chPtr) {
     if (ourread(cfd, &physHeader, PHYS_HDR_SIZE) != PHYS_HDR_SIZE) 
 	return CPIOERR_READ_FAILED;
 
-    if (strncmp(CPIO_CRC_MAGIC, physHeader.magic, strlen(CPIO_CRC_MAGIC)) &&
-	strncmp(CPIO_NEWC_MAGIC, physHeader.magic, strlen(CPIO_NEWC_MAGIC)))
+    if (strncmp(CPIO_CRC_MAGIC, physHeader.magic, sizeof(CPIO_CRC_MAGIC)-1) &&
+	strncmp(CPIO_NEWC_MAGIC, physHeader.magic, sizeof(CPIO_NEWC_MAGIC)-1))
 	return CPIOERR_BAD_MAGIC;
 
     GET_NUM_FIELD(physHeader.inode, chPtr->inode);
@@ -220,9 +237,10 @@ static int getNextHeader(CFD_t * cfd, struct cpioHeader * chPtr) {
 
     GET_NUM_FIELD(physHeader.namesize, nameSize);
 
-    chPtr->path = malloc(nameSize + 1);
+    chPtr->path = xmalloc(nameSize + 1);
     if (ourread(cfd, chPtr->path, nameSize) != nameSize) {
 	free(chPtr->path);
+	chPtr->path = NULL;
 	return CPIOERR_BAD_HEADER;
     }
 
@@ -233,7 +251,8 @@ static int getNextHeader(CFD_t * cfd, struct cpioHeader * chPtr) {
     return 0;
 }
 
-int cpioFileMapCmp(const void * a, const void * b) {
+int cpioFileMapCmp(const void * a, const void * b)
+{
     const struct cpioFileMapping * first = a;
     const struct cpioFileMapping * second = b;
 
@@ -241,7 +260,8 @@ int cpioFileMapCmp(const void * a, const void * b) {
 }
 
 /* This could trash files in the path! I'm not sure that's a good thing */
-static int createDirectory(char * path, mode_t perms) {
+static int createDirectory(char * path, mode_t perms)
+{
     struct stat sb;
 
     if (!lstat(path, &sb)) {
@@ -276,7 +296,8 @@ static int createDirectory(char * path, mode_t perms) {
     return 0;
 }
 
-static int setInfo(struct cpioHeader * hdr) {
+static int setInfo(struct cpioHeader * hdr)
+{
     int rc = 0;
     struct utimbuf stamp;
 
@@ -300,8 +321,9 @@ static int setInfo(struct cpioHeader * hdr) {
     return rc;
 }
 
-static int checkDirectory(char * filename) {
-    static char * lastDir = NULL;
+static int checkDirectory(char * filename)
+{
+    static char * lastDir = NULL;	/* XXX memory leak */
     static int lastDirLength = 0;
     static int lastDirAlloced = 0;
     int length = strlen(filename);
@@ -325,7 +347,7 @@ static int checkDirectory(char * filename) {
 
     if (lastDirAlloced < (length + 1)) {
 	lastDirAlloced = length + 100;
-	lastDir = realloc(lastDir, lastDirAlloced);
+	lastDir = xrealloc(lastDir, lastDirAlloced);	/* XXX memory leak */
     }
 
     strcpy(lastDir, buf);
@@ -345,7 +367,8 @@ static int checkDirectory(char * filename) {
 }
 
 static int expandRegular(CFD_t * cfd, struct cpioHeader * hdr,
-			 cpioCallback cb, void * cbData) {
+			 cpioCallback cb, void * cbData)
+{
     FD_t out;
     char buf[8192];
     int bytesRead;
@@ -407,7 +430,8 @@ static int expandRegular(CFD_t * cfd, struct cpioHeader * hdr,
     return rc;
 }
 
-static int expandSymlink(CFD_t * cfd, struct cpioHeader * hdr) {
+static int expandSymlink(CFD_t * cfd, struct cpioHeader * hdr)
+{
     char buf[2048], buf2[2048];
     struct stat sb;
     int len;
@@ -439,7 +463,8 @@ static int expandSymlink(CFD_t * cfd, struct cpioHeader * hdr) {
     return 0;
 }
 
-static int expandFifo(CFD_t * cfd, struct cpioHeader * hdr) {
+static int expandFifo( /*@unused@*/ CFD_t * cfd, struct cpioHeader * hdr)
+{
     struct stat sb;
 
     if (!lstat(hdr->path, &sb)) {
@@ -455,7 +480,8 @@ static int expandFifo(CFD_t * cfd, struct cpioHeader * hdr) {
     return 0; 
 }
 
-static int expandDevice(CFD_t * cfd, struct cpioHeader * hdr) {
+static int expandDevice( /*@unused@*/ CFD_t * cfd, struct cpioHeader * hdr)
+{
     struct stat sb;
 
     if (!lstat(hdr->path, &sb)) {
@@ -466,22 +492,26 @@ static int expandDevice(CFD_t * cfd, struct cpioHeader * hdr) {
 	    return CPIOERR_UNLINK_FAILED;
     }
 
-    if (mknod(hdr->path, hdr->mode & (~0777), hdr->rdev))
+    if ( /*@-unrecog@*/ mknod(hdr->path, hdr->mode & (~0777), hdr->rdev) /*@=unrecog@*/ )
 	return CPIOERR_MKNOD_FAILED;
     
     return 0;
 }
 
-static void freeLink(struct hardLink * li) {
+static void freeLink( /*@only@*/ struct hardLink * li)
+{
     int i;
 
     for (i = 0; i < li->nlink; i++) {
-	if (li->files[i]) free(li->files[i]);
+	if (li->files[i] == NULL) continue;
+	free((void *)li->files[i]);
+	li->files[i] = NULL;
     }
     free(li->files);
 }
 
-static int createLinks(struct hardLink * li, char ** failedFile) {
+static int createLinks(struct hardLink * li, /*@out@*/const char ** failedFile)
+{
     int i;
     struct stat sb;
 
@@ -491,17 +521,19 @@ static int createLinks(struct hardLink * li, char ** failedFile) {
 
 	if (!lstat(li->files[i], &sb)) {
 	    if (unlink(li->files[i])) {
-		*failedFile = strdup(li->files[i]);
+		if (failedFile)
+		    *failedFile = xstrdup(li->files[i]);
 		return CPIOERR_UNLINK_FAILED;
 	    }
 	}
 
 	if (link(li->files[li->createdPath], li->files[i])) {
-	    *failedFile = strdup(li->files[i]);
+	    if (failedFile)
+		*failedFile = xstrdup(li->files[i]);
 	    return CPIOERR_LINK_FAILED;
 	}
 
-	free(li->files[i]);
+	free((void *)li->files[i]);
 	li->files[i] = NULL;
 	li->linksLeft--;
     }
@@ -509,7 +541,8 @@ static int createLinks(struct hardLink * li, char ** failedFile) {
     return 0;
 }
 
-static int eatBytes(CFD_t * cfd, int amount) {
+static int eatBytes(CFD_t * cfd, int amount)
+{
     char buf[4096];
     int bite;
    
@@ -525,7 +558,8 @@ static int eatBytes(CFD_t * cfd, int amount) {
 
 int cpioInstallArchive(CFD_t *cfd, struct cpioFileMapping * mappings, 
 		       int numMappings, cpioCallback cb, void * cbData,
-		       char ** failedFile) {
+		       const char ** failedFile)
+{
     struct cpioHeader ch;
     int rc = 0;
     int linkNum = 0;
@@ -538,7 +572,8 @@ int cpioInstallArchive(CFD_t *cfd, struct cpioFileMapping * mappings,
     struct hardLink * li = NULL;
 
     cfd->cpioPos = 0;
-    *failedFile = NULL;
+    if (failedFile)
+	*failedFile = NULL;
 
     ch.path = NULL;
     do {
@@ -568,7 +603,7 @@ int cpioInstallArchive(CFD_t *cfd, struct cpioFileMapping * mappings,
 	    if (map) {
 		if (map->mapFlags & CPIO_MAP_PATH) {
 		    free(ch.path);
-		    ch.path = strdup(map->fsPath);
+		    ch.path = xstrdup(map->fsPath);
 		} 
 
 		if (map->mapFlags & CPIO_MAP_MODE)
@@ -587,21 +622,21 @@ int cpioInstallArchive(CFD_t *cfd, struct cpioFileMapping * mappings,
 		    if (li->inode == ch.inode && li->dev == ch.dev) break;
 		}
 
-		if (!li) {
-		    li = malloc(sizeof(*li));
+		if (li == NULL) {
+		    li = xmalloc(sizeof(*li));
 		    li->inode = ch.inode;
 		    li->dev = ch.dev;
 		    li->nlink = ch.nlink;
 		    li->linksLeft = ch.nlink;
 		    li->createdPath = -1;
-		    li->files = calloc(sizeof(char *), li->nlink);
+		    li->files = xcalloc(li->nlink,(sizeof(*li->files)));
 		    li->next = links;
 		    links = li;
 		}
 
 		for (linkNum = 0; linkNum < li->nlink; linkNum++)
 		    if (!li->files[linkNum]) break;
-		li->files[linkNum] = strdup(ch.path);
+		li->files[linkNum] = xstrdup(ch.path);
 	    }
 		
 	    if ((ch.nlink > 1) && S_ISREG(ch.mode) && !ch.size &&
@@ -649,8 +684,8 @@ int cpioInstallArchive(CFD_t *cfd, struct cpioFileMapping * mappings,
 		}
 	    }
 
-	    if (rc && !*failedFile) {
-		*failedFile = strdup(ch.path);
+	    if (rc && failedFile && *failedFile == NULL) {
+		*failedFile = xstrdup(ch.path);
 
 		olderr = errno;
 		unlink(ch.path);
@@ -702,7 +737,8 @@ int cpioInstallArchive(CFD_t *cfd, struct cpioFileMapping * mappings,
 }
 
 static int writeFile(CFD_t *cfd, struct stat sb, struct cpioFileMapping * map, 
-		     size_t * sizeptr, int writeData) {
+		     /*@out@*/size_t * sizep, int writeData)
+{
     struct cpioCrcPhysicalHeader hdr;
     char buf[8192], symbuf[2048];
     dev_t num;
@@ -826,7 +862,8 @@ static int writeFile(CFD_t *cfd, struct stat sb, struct cpioFileMapping * map,
     if ((rc = padoutfd(cfd, &size, 4)))
 	return rc;
 
-    *sizeptr = size;
+    if (sizep)
+	*sizep = size;
 
     return 0;
 }
@@ -834,23 +871,24 @@ static int writeFile(CFD_t *cfd, struct stat sb, struct cpioFileMapping * map,
 static int writeLinkedFile(CFD_t *cfd, struct hardLink * hlink, 
 			   struct cpioFileMapping * mappings,
 			   cpioCallback cb, void * cbData,
-			   size_t * sizeptr,
-			   char ** failedFile) {
+			   /*@out@*/size_t * sizep,
+			   /*@out@*/const char ** failedFile)
+{
     int i, rc;
-    size_t size;
+    size_t size, total;
     struct cpioCallbackInfo cbinfo;
 
-    *sizeptr = 0;
+    total = 0;
 
     for (i = hlink->nlink - 1; i > hlink->linksLeft; i--) {
 	if ((rc = writeFile(cfd, hlink->sb, mappings + hlink->fileMaps[i], 
 			    &size, 0))) {
-	    if (failedFile) *failedFile = 
-		    mappings[hlink->fileMaps[i]].fsPath;
+	    if (failedFile)
+		*failedFile = xstrdup(mappings[hlink->fileMaps[i]].fsPath);
 	    return rc;
 	}
 
-	*sizeptr += size;
+	total += size;
 
 	if (cb) {
 	    cbinfo.file = mappings[i].archivePath;
@@ -861,12 +899,16 @@ static int writeLinkedFile(CFD_t *cfd, struct hardLink * hlink,
     if ((rc = writeFile(cfd, hlink->sb, 
 			mappings + hlink->fileMaps[hlink->linksLeft], 
 			&size, 1))) {
+	if (sizep)
+	    *sizep = total;
 	if (failedFile) 
-	    *failedFile = mappings[hlink->fileMaps[hlink->linksLeft]].fsPath;
+	    *failedFile = xstrdup(mappings[hlink->fileMaps[hlink->linksLeft]].fsPath);
 	return rc;
     }
+    total += size;
 
-    *sizeptr += size;
+    if (sizep)
+	*sizep = total;
 
     if (cb) {
 	cbinfo.file = mappings[i].archivePath;
@@ -878,7 +920,8 @@ static int writeLinkedFile(CFD_t *cfd, struct hardLink * hlink,
 
 int cpioBuildArchive(CFD_t *cfd, struct cpioFileMapping * mappings, 
 		     int numMappings, cpioCallback cb, void * cbData,
-		     unsigned int * archiveSize, char ** failedFile) {
+		     unsigned int * archiveSize, const char ** failedFile)
+{
     size_t size, totalsize = 0;
     int rc;
     int i;
@@ -899,7 +942,8 @@ int cpioBuildArchive(CFD_t *cfd, struct cpioFileMapping * mappings,
 	    rc = lstat(mappings[i].fsPath, &sb);
 
 	if (rc) {
-	    if (failedFile) *failedFile = mappings[i].fsPath;
+	    if (failedFile)
+		*failedFile = xstrdup(mappings[i].fsPath);
 	    return CPIOERR_STAT_FAILED;
 	}
 
@@ -909,7 +953,7 @@ int cpioBuildArchive(CFD_t *cfd, struct cpioFileMapping * mappings,
 		   (hlink->dev != sb.st_dev || hlink->inode != sb.st_ino))
 		hlink = hlink->next;
 	    if (!hlink) {
-		hlink = malloc(sizeof(*hlink));
+		hlink = xmalloc(sizeof(*hlink));
 		hlink->next = hlinkList.next;
 		hlinkList.next = hlink;
 		hlink->sb = sb;		/* structure assignment */
@@ -917,7 +961,7 @@ int cpioBuildArchive(CFD_t *cfd, struct cpioFileMapping * mappings,
 		hlink->inode = sb.st_ino;
 		hlink->nlink = sb.st_nlink;
 		hlink->linksLeft = sb.st_nlink;
-		hlink->fileMaps = malloc(sizeof(*hlink->fileMaps) * 
+		hlink->fileMaps = xmalloc(sizeof(*hlink->fileMaps) * 
 						sb.st_nlink);
 	    }
 
@@ -939,7 +983,8 @@ int cpioBuildArchive(CFD_t *cfd, struct cpioFileMapping * mappings,
 	    }
 	} else {
 	    if ((rc = writeFile(cfd, sb, mappings + i, &size, 1))) {
-		if (failedFile) *failedFile = mappings[i].fsPath;
+		if (failedFile)
+		    *failedFile = xstrdup(mappings[i].fsPath);
 		return rc;
 	    }
 
@@ -996,7 +1041,7 @@ const char * cpioStrerror(int rc)
     switch (rc) {
     default:
 	s = msg + strlen(msg);
-	sprintf(s, _("(error 0x%x)"), rc);
+	sprintf(s, _("(error 0x%x)"), (unsigned)rc);
 	s = NULL;
 	break;
     case CPIOERR_BAD_MAGIC:	s = _("Bad magic");		break;
