@@ -4,7 +4,7 @@
 #include <zlib.h>
 #include <sys/types.h>
 
-#include "rpmio.h"
+#include <rpmio.h>
 
 /* Note the CPIO_CHECK_ERRNO bit is set only if errno is valid. These have to
    be positive numbers or this setting the high bit stuff is a bad idea. */
@@ -45,8 +45,8 @@
 #define CPIO_FOLLOW_SYMLINKS	(1 << 4)  /* only for building */
 
 struct cpioFileMapping {
-    char * archivePath;
-    char * fsPath;
+    /*@dependent@*/ const char * archivePath;
+    /*@dependent@*/ const char * fsPath;
     mode_t finalMode;
     uid_t finalUid;
     gid_t finalGid;
@@ -55,7 +55,7 @@ struct cpioFileMapping {
 
 /* on cpio building, only "file" is filled in */
 struct cpioCallbackInfo {
-    char * file;
+    /*@dependent@*/ const char * file;
     long fileSize;			/* total file size */
     long fileComplete;			/* amount of file unpacked */
     long bytesProcessed;		/* bytes in archive read */
@@ -63,19 +63,26 @@ struct cpioCallbackInfo {
 
 typedef struct CFD {
     union {
-	FD_t	_cfdu_fd;
+	/*@owned@*/FD_t	_cfdu_fd;
 #define	cpioFd	_cfdu._cfdu_fd
-	FILE *	_cfdu_fp;
+	/*@owned@*/FILE *_cfdu_fp;
 #define	cpioFp	_cfdu._cfdu_fp
-	FD_t	_cfdu_gzfd;
+	/*@owned@*/FD_t	_cfdu_gzfd;
 #define	cpioGzFd	_cfdu._cfdu_gzfd
+#if ENABLE_BZIP2_PAYLOAD
+	/*@owned@*/FD_t	_cfdu_bzfd;
+#define	cpioBzFd	_cfdu._cfdu_bzfd
+#endif
     } _cfdu;
     int		cpioPos;
     enum cpioIoType {
 	cpioIoTypeDebug,
 	cpioIoTypeFd,
 	cpioIoTypeFp,
-	cpioIoTypeGzFd
+	cpioIoTypeGzFd,
+#if ENABLE_BZIP2_PAYLOAD
+	cpioIoTypeBzFd,
+#endif
     } cpioIoType;
 } CFD_t;
 
@@ -94,15 +101,15 @@ typedef void (*cpioCallback)(struct cpioCallbackInfo * filespec, void * data);
    user. If *failedFile is non-NULL on return, it should be free()d. */
 int cpioInstallArchive(CFD_t *cfd, struct cpioFileMapping * mappings,
 		       int numMappings, cpioCallback cb, void * cbData,
-		       char ** failedFile);
+		       /*@out@*/const char ** failedFile);
 int cpioBuildArchive(CFD_t *cfd, struct cpioFileMapping * mappings,
 		     int numMappings, cpioCallback cb, void * cbData,
-		     unsigned int * archiveSize, /*@out@*/char ** failedFile);
+		     unsigned int * archiveSize, /*@out@*/const char ** failedFile);
 
 /* This is designed to be qsort/bsearch compatible */
 int cpioFileMapCmp(const void * a, const void * b);
 
-const char *cpioStrerror(int rc);
+/*@observer@*/ const char *cpioStrerror(int rc);
 
 #ifdef __cplusplus
 }
