@@ -1,43 +1,10 @@
 #include "system.h"
 
-#include "rpmlib.h"
+#include <rpmlib.h>
 
-static char * permsFormat(int_32 type, const void * data, 
-		          char * formatPrefix, int padding, int element);
-static char * depflagsFormat(int_32 type, const void * data, 
-		             char * formatPrefix, int padding, int element);
-static char * triggertypeFormat(int_32 type, const void * data, 
-		         char * formatPrefix, int padding, int element);
-static char * fflagsFormat(int_32 type, const void * data, 
-		           char * formatPrefix, int padding, int element);
-static int fsnamesTag(Header h, int_32 * type, void ** data, int_32 * count,
-		      int * freeData);
-static int fssizesTag(Header h, int_32 * type, void ** data, int_32 * count,
-		      int * freeData);
-static int instprefixTag(Header h, int_32 * type, void ** data, int_32 * count,
-		         int * freeData);
-static int triggercondsTag(Header h, int_32 * type, void ** data, 
-			   int_32 * count, int * freeData);
-static int triggertypeTag(Header h, int_32 * type, void ** data, 
-			  int_32 * count, int * freeData);
-static char * permsString(int mode);
-
-const struct headerSprintfExtension rpmHeaderFormats[] = {
-    { HEADER_EXT_TAG, "RPMTAG_FSSIZES", { fssizesTag } },
-    { HEADER_EXT_TAG, "RPMTAG_FSNAMES", { fsnamesTag } },
-    { HEADER_EXT_TAG, "RPMTAG_INSTALLPREFIX", { instprefixTag } },
-    { HEADER_EXT_TAG, "RPMTAG_TRIGGERCONDS", { triggercondsTag } },
-    { HEADER_EXT_TAG, "RPMTAG_TRIGGERTYPE", { triggertypeTag } },
-    { HEADER_EXT_FORMAT, "depflags", { depflagsFormat } },
-    { HEADER_EXT_FORMAT, "fflags", { fflagsFormat } },
-    { HEADER_EXT_FORMAT, "perms", { permsFormat } },
-    { HEADER_EXT_FORMAT, "permissions", { permsFormat } },
-    { HEADER_EXT_FORMAT, "triggertype", { triggertypeFormat } },
-    { HEADER_EXT_MORE, NULL, { (void *) headerDefaultFormats } }
-} ;
-
-static char * permsString(int mode) {
-    char * perms = malloc(11);
+static char * permsString(int mode)
+{
+    char * perms = xmalloc(11);
 
     strcpy(perms, "-----------");
    
@@ -88,32 +55,35 @@ static char * permsString(int mode) {
 }
 
 static char * triggertypeFormat(int_32 type, const void * data, 
-		         char * formatPrefix, int padding, int element) {
+	/*@unused@*/char * formatPrefix, /*@unused@*/int padding,
+	/*@unused@*/int element)
+{
     const int_32 * item = data;
     char * val;
 
     if (type != RPM_INT32_TYPE) {
-	val = malloc(20);
+	val = xmalloc(20);
 	strcpy(val, _("(not a number)"));
     } else if (*item & RPMSENSE_TRIGGERIN) {
-	val = strdup("in");
+	val = xstrdup("in");
     } else {
-	val = strdup("un");
+	val = xstrdup("un");
     }
 
     return val;
 }
 
 static char * permsFormat(int_32 type, const void * data, 
-		         char * formatPrefix, int padding, int element) {
+		char * formatPrefix, int padding, /*@unused@*/int element)
+{
     char * val;
     char * buf;
 
     if (type != RPM_INT32_TYPE) {
-	val = malloc(20);
+	val = xmalloc(20);
 	strcpy(val, _("(not a number)"));
     } else {
-	val = malloc(15 + padding);
+	val = xmalloc(15 + padding);
 	strcat(formatPrefix, "s");
 	buf = permsString(*((int_32 *) data));
 	sprintf(val, formatPrefix, buf);
@@ -124,13 +94,14 @@ static char * permsFormat(int_32 type, const void * data,
 }
 
 static char * fflagsFormat(int_32 type, const void * data, 
-		         char * formatPrefix, int padding, int element) {
+		char * formatPrefix, int padding, /*@unused@*/int element)
+{
     char * val;
     char buf[15];
     int anint = *((int_32 *) data);
 
     if (type != RPM_INT32_TYPE) {
-	val = malloc(20);
+	val = xmalloc(20);
 	strcpy(val, _("(not a number)"));
     } else {
 	buf[0] = '\0';
@@ -147,7 +118,7 @@ static char * fflagsFormat(int_32 type, const void * data,
 	if (anint & RPMFILE_GHOST)
 	    strcat(buf, "g");
 
-	val = malloc(5 + padding);
+	val = xmalloc(5 + padding);
 	strcat(formatPrefix, "s");
 	sprintf(val, formatPrefix, buf);
     }
@@ -156,13 +127,14 @@ static char * fflagsFormat(int_32 type, const void * data,
 }
 
 static char * depflagsFormat(int_32 type, const void * data, 
-		         char * formatPrefix, int padding, int element) {
+		char * formatPrefix, int padding, /*@unused@*/int element)
+{
     char * val;
     char buf[10];
     int anint = *((int_32 *) data);
 
     if (type != RPM_INT32_TYPE) {
-	val = malloc(20);
+	val = xmalloc(20);
 	strcpy(val, _("(not a number)"));
     } else {
 	buf[0] = '\0';
@@ -173,10 +145,8 @@ static char * depflagsFormat(int_32 type, const void * data,
 	    strcat(buf, ">");
 	if (anint & RPMSENSE_EQUAL)
 	    strcat(buf, "=");
-	if (anint & RPMSENSE_SERIAL)
-	    strcat(buf, "S");
 
-	val = malloc(5 + padding);
+	val = xmalloc(5 + padding);
 	strcat(formatPrefix, "s");
 	sprintf(val, formatPrefix, buf);
     }
@@ -184,8 +154,9 @@ static char * depflagsFormat(int_32 type, const void * data,
     return val;
 }
 
-static int fsnamesTag(Header h, int_32 * type, void ** data, int_32 * count,
-		      int * freeData) {
+static int fsnamesTag( /*@unused@*/ Header h, int_32 * type, void ** data,
+	int_32 * count, int * freeData)
+{
     const char ** list;
 
     if (rpmGetFilesystemList(&list, count)) {
@@ -201,7 +172,8 @@ static int fsnamesTag(Header h, int_32 * type, void ** data, int_32 * count,
 }
 
 static int instprefixTag(Header h, int_32 * type, void ** data, int_32 * count,
-		      int * freeData) {
+		      int * freeData)
+{
     char ** array;
 
     if (headerGetEntry(h, RPMTAG_INSTALLPREFIX, type, data, count)) {
@@ -209,7 +181,7 @@ static int instprefixTag(Header h, int_32 * type, void ** data, int_32 * count,
 	return 0;
     } else if (headerGetEntry(h, RPMTAG_INSTPREFIXES, NULL, (void **) &array, 
 			      count)) {
-	*((char **) data) = strdup(array[0]);
+	*((char **) data) = xstrdup(array[0]);
 	*freeData = 1;
 	*type = RPM_STRING_TYPE;
 	free(array);
@@ -220,18 +192,22 @@ static int instprefixTag(Header h, int_32 * type, void ** data, int_32 * count,
 }
 
 static int fssizesTag(Header h, int_32 * type, void ** data, int_32 * count,
-		      int * freeData) {
+		      int * freeData)
+{
     const char ** filenames;
     int_32 * filesizes;
     uint_32 * usages;
     int numFiles;
 
-    if (headerGetEntry(h, RPMTAG_FILENAMES, NULL, (void **) &filenames, NULL)) 
-	headerGetEntry(h, RPMTAG_FILESIZES, NULL, (void **) &filesizes, 
-		       &numFiles);
-    else
+    if (!headerGetEntry(h, RPMTAG_FILENAMES, NULL, (void **) &filenames, NULL)) 
 	filenames = NULL;
 
+    if (!headerGetEntry(h, RPMTAG_FILESIZES, NULL, (void **) &filesizes, 
+		       &numFiles)) {
+	filesizes = NULL;
+	numFiles = 0;
+    }
+
     if (rpmGetFilesystemList(NULL, count)) {
 	return 1;
     }
@@ -239,9 +215,9 @@ static int fssizesTag(Header h, int_32 * type, void ** data, int_32 * count,
     *type = RPM_INT32_TYPE;
     *freeData = 1;
 
-    if (!filenames) {
-	*data = usages = malloc(sizeof(usages) * (*count));
-	memset(usages, 0, sizeof(usages) * (*count));
+    if (filenames == NULL) {
+	usages = xcalloc((*count), sizeof(usages));
+	*data = usages;
 
 	return 0;
     }
@@ -254,8 +230,9 @@ static int fssizesTag(Header h, int_32 * type, void ** data, int_32 * count,
     return 0;
 }
 
-static int triggercondsTag(Header h, int_32 * type, void ** data, 
-			   int_32 * count, int * freeData) {
+static int triggercondsTag(Header h, /*@out@*/int_32 * type, /*@out@*/void ** data, 
+			   /*@out@*/int_32 * count, /*@out@*/int * freeData)
+{
     int_32 * indices, * flags;
     char ** names, ** versions;
     int numNames, numScripts;
@@ -278,17 +255,16 @@ static int triggercondsTag(Header h, int_32 * type, void ** data,
     free(s);
 
     *freeData = 1;
-    *data = conds = malloc(sizeof(char * ) * numScripts);
+    *data = conds = xmalloc(sizeof(char * ) * numScripts);
     *count = numScripts;
     *type = RPM_STRING_ARRAY_TYPE;
     for (i = 0; i < numScripts; i++) {
-	chptr = malloc(1);
-	*chptr = '\0';
+	chptr = xstrdup("");
 
 	for (j = 0; j < numNames; j++) {
 	    if (indices[j] != i) continue;
 
-	    item = malloc(strlen(names[j]) + strlen(versions[j]) + 20);
+	    item = xmalloc(strlen(names[j]) + strlen(versions[j]) + 20);
 	    if (flags[j] & RPMSENSE_SENSEMASK) {
 		buf[0] = '%', buf[1] = '\0';
 		flagsStr = depflagsFormat(RPM_INT32_TYPE, flags, buf,
@@ -299,7 +275,7 @@ static int triggercondsTag(Header h, int_32 * type, void ** data,
 		strcpy(item, names[j]);
 	    }
 
-	    chptr = realloc(chptr, strlen(chptr) + strlen(item) + 5);
+	    chptr = xrealloc(chptr, strlen(chptr) + strlen(item) + 5);
 	    if (*chptr) strcat(chptr, ", ");
 	    strcat(chptr, item);
 	    free(item);
@@ -314,8 +290,9 @@ static int triggercondsTag(Header h, int_32 * type, void ** data,
     return 0;
 }
 
-static int triggertypeTag(Header h, int_32 * type, void ** data, 
-			   int_32 * count, int * freeData) {
+static int triggertypeTag(Header h, int_32 * type, /*@out@*/void ** data, 
+			   int_32 * count, int * freeData)
+{
     int_32 * indices, * flags;
     char ** conds, ** s;
     int i, j;
@@ -333,7 +310,7 @@ static int triggertypeTag(Header h, int_32 * type, void ** data,
     free(s);
 
     *freeData = 1;
-    *data = conds = malloc(sizeof(char * ) * numScripts);
+    *data = conds = xmalloc(sizeof(char * ) * numScripts);
     *count = numScripts;
     *type = RPM_STRING_ARRAY_TYPE;
     for (i = 0; i < numScripts; i++) {
@@ -341,14 +318,28 @@ static int triggertypeTag(Header h, int_32 * type, void ** data,
 	    if (indices[j] != i) continue;
 
 	    if (flags[j] & RPMSENSE_TRIGGERIN)
-		conds[i] = strdup("in");
+		conds[i] = xstrdup("in");
 	    else if (flags[j] & RPMSENSE_TRIGGERUN)
-		conds[i] = strdup("un");
+		conds[i] = xstrdup("un");
 	    else
-		conds[i] = strdup("postun");
+		conds[i] = xstrdup("postun");
 	    break;
 	}
     }
 
     return 0;
 }
+
+const struct headerSprintfExtension rpmHeaderFormats[] = {
+    { HEADER_EXT_TAG, "RPMTAG_FSSIZES", { fssizesTag } },
+    { HEADER_EXT_TAG, "RPMTAG_FSNAMES", { fsnamesTag } },
+    { HEADER_EXT_TAG, "RPMTAG_INSTALLPREFIX", { instprefixTag } },
+    { HEADER_EXT_TAG, "RPMTAG_TRIGGERCONDS", { triggercondsTag } },
+    { HEADER_EXT_TAG, "RPMTAG_TRIGGERTYPE", { triggertypeTag } },
+    { HEADER_EXT_FORMAT, "depflags", { depflagsFormat } },
+    { HEADER_EXT_FORMAT, "fflags", { fflagsFormat } },
+    { HEADER_EXT_FORMAT, "perms", { permsFormat } },
+    { HEADER_EXT_FORMAT, "permissions", { permsFormat } },
+    { HEADER_EXT_FORMAT, "triggertype", { triggertypeFormat } },
+    { HEADER_EXT_MORE, NULL, { (void *) headerDefaultFormats } }
+} ;
