@@ -1,7 +1,6 @@
 #include "system.h"
 
-#include "rpmlib.h"
-
+#include <rpmlib.h>
 #include "fprint.h"
 
 struct lookupCache {
@@ -13,22 +12,17 @@ struct lookupCache {
     ino_t ino;
 };
 
-static fingerPrint doLookup(const char * fullName, int scareMemory, 
-			    struct lookupCache * cache);
-
-static int strCompare(const void * a, const void * b) {
+static int strCompare(const void * a, const void * b)
+{
     const char * const * one = a;
-    const char * const * two = a;
+    const char * const * two = b;
 
     return strcmp(*one, *two);
 }
 
-fingerPrint fpLookup(const char * fullName, int scareMemory) {
-    return doLookup(fullName, scareMemory, NULL);
-}
-
 static fingerPrint doLookup(const char * fullName, int scareMemory, 
-			    struct lookupCache * cache) {
+			    struct lookupCache * cache)
+{
     char dir[PATH_MAX];
     const char * chptr1;
     char * end, * bn;
@@ -58,7 +52,7 @@ static fingerPrint doLookup(const char * fullName, int scareMemory,
 		if (scareMemory)
 		    fp.basename = chptr1;
 		else
-		    fp.basename = strdup(chptr1);
+		    fp.basename = xstrdup(chptr1);
 		fp.ino = cache->ino;
 		fp.dev = cache->dev;
 		return fp;
@@ -77,7 +71,8 @@ static fingerPrint doLookup(const char * fullName, int scareMemory,
 
 	/* if the current directory doesn't exist, we might fail. 
 	   oh well. likewise if it's too long.  */
-	if (realpath(".", dir) != NULL) {
+	dir[0] = '\0';
+	if ( /*@-unrecog@*/ realpath(".", dir) /*@=unrecog@*/ != NULL) {
 	    char *s = alloca(strlen(dir) + strlen(fullName) + 2);
 	    sprintf(s, "%s/%s", dir, fullName);
 	    fullName = chptr1 = s;
@@ -101,7 +96,7 @@ static fingerPrint doLookup(const char * fullName, int scareMemory,
 	    if (scareMemory)
 		fp.basename = chptr1;
 	    else
-		fp.basename = strdup(chptr1);
+		fp.basename = xstrdup(chptr1);	/* XXX memory leak */
 	    fp.ino = sb.st_ino;
 	    fp.dev = sb.st_dev;
 
@@ -131,7 +126,8 @@ static fingerPrint doLookup(const char * fullName, int scareMemory,
     return fp;
 }
 
-unsigned int fpHashFunction(const void * key) {
+unsigned int fpHashFunction(const void * key)
+{
     const fingerPrint * fp = key;
     unsigned int hash = 0;
     char ch;
@@ -141,19 +137,26 @@ unsigned int fpHashFunction(const void * key) {
     chptr = fp->basename;
     while (*chptr) ch ^= *chptr++;
 
-    hash |= ch << 24;
-    hash |= (((fp->dev >> 8) ^ fp->dev) & 0xFF) << 16;
+    hash |= ((unsigned)ch) << 24;
+    hash |= (((((unsigned)fp->dev) >> 8) ^ fp->dev) & 0xFF) << 16;
     hash |= fp->ino & 0xFFFF;
     
     return hash;
 }
 
-int fpEqual(const void * key1, const void * key2) {
+int fpEqual(const void * key1, const void * key2)
+{
     return FP_EQUAL(*((const fingerPrint *) key1), *((fingerPrint *) key2));
 }
 
+fingerPrint fpLookup(const char * fullName, int scareMemory)
+{
+    return doLookup(fullName, scareMemory, NULL);
+}
+
 void fpLookupList(const char ** fullNames, fingerPrint * fpList, int numItems,
-		  int alreadySorted) {
+		  int alreadySorted)
+{
     int i, j;
     struct lookupCache cache;
     int maxLen = 0;
@@ -172,6 +175,8 @@ void fpLookupList(const char ** fullNames, fingerPrint * fpList, int numItems,
     cache.matchLength = 0;
     cache.pathsStripped = 0;    
     cache.stripLength = 0;
+    cache.dev = 0;
+    cache.ino = 0;
 
     for (i = 0; i < numItems; i++) {
 	fpList[i] = doLookup(fullNames[i], 1, &cache);
