@@ -7,14 +7,6 @@ typedef struct fingerprint_s {
     const char * basename;
 } fingerPrint;
 
-/* Be carefull with the memory... assert(*fullName == '/' || !scareMemory) */
-fingerPrint fpLookup(const char * fullName, int scareMemory);
-unsigned int fpHashFunction(const void * string);
-int fpEqual(const void * key1, const void * key2);
-/* scareMemory is assumed here! */
-void fpLookupList(const char ** fullNames, fingerPrint * fpList, int numItems,
-		  int alreadySorted);
-
 /* only if !scarceMemory */
 #define fpFree(a) free((void *)(a).basename)
 
@@ -23,4 +15,20 @@ void fpLookupList(const char ** fullNames, fingerPrint * fpList, int numItems,
 			        ((a).ino == (b).ino) && \
 			        !strcmp((a).basename, (b).basename)))
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/* Be carefull with the memory... assert(*fullName == '/' || !scareMemory) */
+fingerPrint fpLookup(const char * fullName, int scareMemory);
+unsigned int fpHashFunction(const void * string);
+int fpEqual(const void * key1, const void * key2);
+/* scareMemory is assumed here! */
+void fpLookupList(const char ** fullNames, fingerPrint * fpList, int numItems,
+		  int alreadySorted);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif
