@@ -1,18 +1,16 @@
 #include "system.h"
 
-#include "rpmlib.h"
-#include "rpmmacro.h"
+#include <rpmlib.h>
+#include <rpmmacro.h>
 
 struct fsinfo {
-    char * mntPoint;
+    /*@only@*/ const char * mntPoint;
     dev_t dev;
 };
 
-static struct fsinfo * filesystems;
-static const char ** fsnames;
-static int numFilesystems;
-
-static int getFilesystemList(void);
+/*@only@*/ /*@null@*/ static struct fsinfo * filesystems = NULL;
+/*@only@*/ /*@null@*/ static const char ** fsnames = NULL;
+static int numFilesystems = 0;
 
 #if HAVE_MNTCTL
 
@@ -27,7 +25,8 @@ static int getFilesystemList(void);
  */
 int mntctl(int command, int size, char *buffer);
 
-static int getFilesystemList(void) {
+static int getFilesystemList(void)
+{
     int size;
     void * buf;
     struct vmount * vm;
@@ -60,13 +59,13 @@ static int getFilesystemList(void) {
 
     numFilesystems = num;
 
-    filesystems = malloc(sizeof(*filesystems) * (numFilesystems + 1));
-    fsnames = malloc(sizeof(char *) * (numFilesystems + 1));
+    filesystems = xcalloc((numFilesystems + 1), sizeof(*filesystems));
+    fsnames = xcalloc((numFilesystems + 1), sizeof(char *));
     
     for (vm = buf, i = 0; i < num; i++) {
 	char *fsn;
 	fsnameLength = vm->vmt_data[VMT_STUB].vmt_size;
-	fsn = malloc(fsnameLength + 1);
+	fsn = xmalloc(fsnameLength + 1);
 	strncpy(fsn, (char *)vm + vm->vmt_data[VMT_STUB].vmt_off, 
 		fsnameLength);
 
@@ -76,12 +75,12 @@ static int getFilesystemList(void) {
 	    rpmError(RPMERR_STAT, _("failed to stat %s: %s"), fsnames[i],
 			strerror(errno));
 
-	    for (i = 0; i < num; i++)
-		free(filesystems[i].mntPoint);
 	    free(filesystems);
-	    free(fsnames);
-
 	    filesystems = NULL;
+	    for (i = 0; i < num; i++)
+		free(fsnames[i]);
+	    free(fsnames);
+	    fsnames = NULL;
 	}
 	
 	filesystems[i].dev = sb.st_dev;
@@ -96,7 +95,8 @@ static int getFilesystemList(void) {
     return 0;
 }
 #else 
-static int getFilesystemList(void) {
+static int getFilesystemList(void)
+{
     int numAlloced = 10;
     int num = 0;
     struct stat sb;
@@ -111,6 +111,8 @@ static int getFilesystemList(void) {
     int nextMount = 0;
 #   endif
 
+    rpmMessage(RPMMESS_DEBUG, _("getting list of mounted filesystems\n"));
+
 #   if GETMNTENT_ONE || GETMNTENT_TWO
 	mtab = fopen(MOUNTED, "r");
 	if (!mtab) {
@@ -122,7 +124,7 @@ static int getFilesystemList(void) {
 	getmntinfo_r(&mounts, flags, &mntCount, &bufSize);
 #   endif
 
-    filesystems = malloc(sizeof(*filesystems) * (numAlloced + 1));
+    filesystems = xcalloc((numAlloced + 1), sizeof(*filesystems));	/* XXX memory leak */
 
     while (1) {
 #	if GETMNTENT_ONE
@@ -145,7 +147,7 @@ static int getFilesystemList(void) {
 			strerror(errno));
 
 	    for (i = 0; i < num; i++)
-		free(filesystems[i].mntPoint);
+		xfree(filesystems[i].mntPoint);
 	    free(filesystems);
 
 	    filesystems = NULL;
@@ -154,12 +156,13 @@ static int getFilesystemList(void) {
 
 	if (num == numAlloced) {
 	    numAlloced += 10;
-	    filesystems = realloc(filesystems, 
+	    filesystems = xrealloc(filesystems, 
 				  sizeof(*filesystems) * (numAlloced + 1));
 	}
 
 	filesystems[num].dev = sb.st_dev;
-	filesystems[num++].mntPoint = strdup(mntdir);
+	filesystems[num].mntPoint = xstrdup(mntdir);	/* XXX memory leak */
+	num++;
     }
 
 #   if GETMNTENT_ONE || GETMNTENT_TWO
@@ -168,9 +171,10 @@ static int getFilesystemList(void) {
 	free(mounts);
 #   endif
 
+    filesystems[num].dev = 0;
     filesystems[num].mntPoint = NULL;
 
-    fsnames = malloc(sizeof(*fsnames) * (num + 1));
+    fsnames = xcalloc((num + 1), sizeof(*fsnames));	/* XXX memory leak */
     for (i = 0; i < num; i++)
 	fsnames[i] = filesystems[i].mntPoint;
     fsnames[num] = NULL;
@@ -181,7 +185,8 @@ static int getFilesystemList(void) {
 }
 #endif
 
-int rpmGetFilesystemList(const char *** listptr, int * num) {
+int rpmGetFilesystemList(const char *** listptr, int * num)
+{
     if (!fsnames) 
 	if (getFilesystemList())
 	    return 1;
@@ -193,13 +198,15 @@ int rpmGetFilesystemList(const char *** listptr, int * num) {
 }
 
 int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes, int numFiles,
-			  uint_32 ** usagesPtr, int flags) {
+			  uint_32 ** usagesPtr, /*@unused@*/int flags)
+{
     int_32 * usages;
     int i, len, j;
     char * buf, * dirName;
     char * chptr;
     int maxLen;
-    char * lastDir, * sourceDir;
+    char * lastDir;
+    const char * sourceDir;
     int lastfs = 0;
     int lastDev = -1;		/* I hope nobody uses -1 for a st_dev */
     struct stat sb;
@@ -208,9 +215,11 @@ int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes, int numFiles
 	if (getFilesystemList())
 	    return 1;
 
-    usages = calloc(numFilesystems, sizeof(usages));
+    usages = xcalloc(numFilesystems, sizeof(usages));
 
-    maxLen = 0;
+    sourceDir = rpmGetPath("", "/%{_sourcedir}", NULL);
+
+    maxLen = strlen(sourceDir);
     for (i = 0; i < numFiles; i++) {
 	len = strlen(fileList[i]);
 	if (maxLen < len) maxLen = len;
@@ -221,8 +230,6 @@ int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes, int numFiles
     dirName = alloca(maxLen + 1);
     *lastDir = '\0';
 
-    sourceDir = rpmGetPath("", "/%{_sourcedir}", NULL);
-
     /* cut off last filename */
     for (i = 0; i < numFiles; i++) {
 	if (*fileList[i] == '/') {
@@ -235,7 +242,7 @@ int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes, int numFiles
 		*chptr-- = '\0';
 	} else {
 	    /* this should only happen for source packages (gulp) */
-	    buf = sourceDir;
+	    strcpy(buf,  sourceDir);
 	}
 
 	if (strcmp(lastDir, buf)) {
@@ -278,7 +285,7 @@ int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes, int numFiles
 	usages[lastfs] += fssizes[i];
     }
 
-    if (sourceDir) free(sourceDir);
+    if (sourceDir) xfree(sourceDir);
 
     *usagesPtr = usages;
 
