@@ -4,6 +4,8 @@
 #include <stdarg.h>
 
 #define	isblank(_c)	((_c) == ' ' || (_c) == '\t')
+#define	iseol(_c)	((_c) == '\n' || (_c) == '\r')
+
 #define	STREQ(_t, _f, _fn)	((_fn) == (sizeof(_t)-1) && !strncmp((_t), (_f), (_fn)))
 #define	FREE(_x)	{ if (_x) free((void *)_x); (_x) = NULL; }
 
@@ -26,10 +28,10 @@ typedef	int FD_t;
 #define	fdRead		read
 #define	fdClose		close
 #else
-#include "rpmlib.h"
+#include <rpmlib.h>
 #endif
 
-#include "rpmmacro.h"
+#include <rpmmacro.h>
 
 struct MacroContext globalMacroContext;
 
@@ -45,7 +47,6 @@ typedef struct MacroBuf {
 } MacroBuf;
 
 #define SAVECHAR(_mb, _c) { *(_mb)->t = (_c), (_mb)->t++, (_mb)->nb--; }
-#define	DELECHAR(_mb, _c) { if ((_mb)->t[-1] == (_c)) *((_mb)->t--) = '\0', (_mb)->nb++; }
 
 static int expandMacro(MacroBuf *mb);
 
@@ -87,12 +88,13 @@ expandMacroTable(MacroContext *mc)
 {
 	if (mc->macroTable == NULL) {
 		mc->macrosAllocated = MACRO_CHUNK_SIZE;
-		mc->macroTable = (MacroEntry **)malloc(sizeof(*(mc->macroTable)) *
-				mc->macrosAllocated);
+		mc->macroTable = (MacroEntry **)
+		    xmalloc(sizeof(*(mc->macroTable)) * mc->macrosAllocated);
 		mc->firstFree = 0;
 	} else {
 		mc->macrosAllocated += MACRO_CHUNK_SIZE;
-		mc->macroTable = (MacroEntry **)realloc(mc->macroTable, sizeof(*(mc->macroTable)) *
+		mc->macroTable = (MacroEntry **)
+		    xrealloc(mc->macroTable, sizeof(*(mc->macroTable)) *
 				mc->macrosAllocated);
 	}
 	memset(&mc->macroTable[mc->firstFree], 0, MACRO_CHUNK_SIZE * sizeof(*(mc->macroTable)));
@@ -101,8 +103,18 @@ expandMacroTable(MacroContext *mc)
 static void
 sortMacroTable(MacroContext *mc)
 {
+	int i;
+
 	qsort(mc->macroTable, mc->firstFree, sizeof(*(mc->macroTable)),
 		compareMacroName);
+
+	/* Empty pointers are now at end of table. Reset first free index. */
+	for (i = 0; i < mc->firstFree; i++) {
+		if (mc->macroTable[i] != NULL)
+			continue;
+		mc->firstFree = i;
+		break;
+	}
 }
 
 void
@@ -121,6 +133,7 @@ dumpMacroTable(MacroContext *mc, FILE *f)
 	for (i = 0; i < mc->firstFree; i++) {
 		MacroEntry *me;
 		if ((me = mc->macroTable[i]) == NULL) {
+			/* XXX this should never happen */
 			nempty++;
 			continue;
 		}
@@ -172,25 +185,30 @@ rdcl(char *buf, size_t size, FILE *fp, int escapes)
 {
 	char *q = buf;
 	size_t nb = 0;
+	size_t nread = 0;
 
+	*q = '\0';
 	do {
-		*q = '\0';			/* next char in buf */
-		nb = 0;
 		if (fgets(q, size, fp) == NULL)	/* read next line */
 			break;
 		nb = strlen(q);
-		q += nb - 1;			/* last char in buf */
-		*q-- = '\0';			/* trim newline */
-		if (nb < 2 || *q != '\\')	/* continue? */
-			break;
-		if (escapes)			/* copy escape too */
-			q++;
-		else
+		nread += nb;
+		for (q += nb - 1; nb > 0 && iseol(*q); q--)
 			nb--;
-		*q++ = '\n';			/* next char in buf */
+		if (!(nb > 0 && *q == '\\')) {	/* continue? */
+			*(++q) = '\0';		/* trim trailing \r, \n */
+			break;
+		}
+		if (escapes) {			/* copy escape too */
+			q++;
+			nb++;
+		}
 		size -= nb;
+		if (*q == '\r')			/* XXX avoid \r madness */
+			*q = '\n';
+		*(++q) = '\0';			/* next char in buf */
 	} while (size > 0);
-	return (nb > 0 ? buf : NULL);
+	return (nread > 0 ? buf : NULL);
 }
 
 /* Return text between pl and matching pr */
@@ -230,9 +248,9 @@ printMacro(MacroBuf *mb, const char *s, const char *se)
 	if (s[-1] == '{')
 		s--;
 
-	/* If not end-of-string, print only to newline or end-of-string ... */
-	if (*(senl = se) != '\0' && (senl = strchr(senl, '\n')) == NULL)
-		senl = se + strlen(se);
+	/* Print only to first end-of-line (or end-of-string). */
+	for (senl = se; *senl && !iseol(*senl); senl++)
+		;
 
 	/* Limit trailing non-trace output */
 	choplen = 61 - (2 * mb->depth);
@@ -262,11 +280,13 @@ printExpansion(MacroBuf *mb, const char *t, const char *te)
 	}
 
 	/* Shorten output which contains newlines */
-	while (te > t && te[-1] == '\n')
+	while (te > t && iseol(te[-1]))
 		te--;
 	ellipsis = "";
 	if (mb->depth > 0) {
 		const char *tenl;
+
+		/* Skip to last line of expansion */
 		while ((tenl = strchr(t, '\n')) && tenl < te)
 			t = ++tenl;
 
@@ -289,7 +309,7 @@ printExpansion(MacroBuf *mb, const char *t, const char *te)
 		(_s)++;
 
 #define	SKIPNONBLANK(_s, _c)	\
-	while (((_c) = *(_s)) && !(isblank(_c) || c == '\n')) \
+	while (((_c) = *(_s)) && !(isblank(_c) || iseol(_c))) \
 		(_s)++;
 
 #define	COPYNAME(_ne, _s, _c)	\
@@ -306,7 +326,7 @@ printExpansion(MacroBuf *mb, const char *t, const char *te)
     }
 
 #define	COPYBODY(_be, _s, _c)	\
-    {	while(((_c) = *(_s)) && (_c) != '\n') { \
+    {	while(((_c) = *(_s)) && !iseol(_c)) { \
 		if ((_c) == '\\') \
 			(_s)++;	\
 		*(_be)++ = *(_s)++; \
@@ -399,7 +419,11 @@ doShellEscape(MacroBuf *mb, const char *cmd, size_t clen)
 		SAVECHAR(mb, c);
 	pclose(shf);
 
-	DELECHAR(mb, '\n');	/* XXX delete trailing newline (if any) */
+	/* XXX delete trailing \r \n */
+	while (iseol(mb->t[-1])) {
+		*(mb->t--) = '\0';
+		mb->nb++;
+	}
 	return 0;
 }
 
@@ -444,13 +468,13 @@ doDefine(MacroBuf *mb, const char *se, int level, int expandbody)
 		COPYBODY(be, s, c);
 
 		/* Trim trailing blanks/newlines */
-		while (--be >= b && (c = *be) && (isblank(c) || c == '\n'))
+		while (--be >= b && (c = *be) && (isblank(c) || iseol(c)))
 			;
 		*(++be) = '\0';	/* one too far */
 	}
 
 	/* Move scan over body */
-	if (*s == '\n')
+	while (iseol(*s))
 		s++;
 	se = s;
 
@@ -491,7 +515,7 @@ doUndefine(MacroContext *mc, const char *se)
 	COPYNAME(ne, s, c);
 
 	/* Move scan over body */
-	if (*s == '\n')
+	while (iseol(*s))
 		s++;
 	se = s;
 
@@ -524,12 +548,12 @@ static void
 pushMacro(MacroEntry **mep, const char *n, const char *o, const char *b, int level)
 {
 	MacroEntry *prev = (*mep ? *mep : NULL);
-	MacroEntry *me = (MacroEntry *) malloc(sizeof(*me));
+	MacroEntry *me = (MacroEntry *) xmalloc(sizeof(*me));
 
 	me->prev = prev;
-	me->name = (prev ? prev->name : strdup(n));
-	me->opts = (o ? strdup(o) : NULL);
-	me->body = strdup(b ? b : "");
+	me->name = (prev ? prev->name : xstrdup(n));
+	me->opts = (o ? xstrdup(o) : NULL);
+	me->body = xstrdup(b ? b : "");
 	me->used = 0;
 	me->level = level;
 	*mep = me;
@@ -554,32 +578,42 @@ static void
 freeArgs(MacroBuf *mb)
 {
 	MacroContext *mc = mb->mc;
-	int c;
+	int ndeleted = 0;
+	int i;
 
 	/* Delete dynamic macro definitions */
-	for (c = 0; c < mc->firstFree; c++) {
-		MacroEntry *me;
+	for (i = 0; i < mc->firstFree; i++) {
+		MacroEntry **mep, *me;
 		int skiptest = 0;
-		if ((me = mc->macroTable[c]) == NULL)
+		mep = &mc->macroTable[i];
+		me = *mep;
+
+		if (me == NULL)		/* XXX this should never happen */
 			continue;
 		if (me->level < mb->depth)
 			continue;
 		if (strlen(me->name) == 1 && strchr("#*0", *me->name)) {
 			if (*me->name == '*' && me->used > 0)
 				skiptest = 1;
-			; /* XXX skip test for %# %* %0 */
+			/* XXX skip test for %# %* %0 */
 		} else if (!skiptest && me->used <= 0) {
 #if NOTYET
 			rpmError(RPMERR_BADSPEC, _("Macro %%%s (%s) was not used below level %d"),
 				me->name, me->body, me->level);
 #endif
 		}
-		popMacro(&mc->macroTable[c]);
+		popMacro(mep);
+		if (!(mep && *mep))
+			ndeleted++;
 	}
+
+	/* If any deleted macros, sort macro table */
+	if (ndeleted)
+		sortMacroTable(mc);
 }
 
 static const char *
-grabArgs(MacroBuf *mb, const MacroEntry *me, const char *se)
+grabArgs(MacroBuf *mb, const MacroEntry *me, const char *se, char lastc)
 {
     char buf[BUFSIZ], *b, *be;
     char aname[16];
@@ -590,6 +624,7 @@ grabArgs(MacroBuf *mb, const MacroEntry *me, const char *se)
     const char **optv;
     int opte;
     int c;
+    int saveoptind;	/* XXX optind must be saved on non-linux */
 
     /* Copy macro name as argv[0] */
     argc = 0;
@@ -601,20 +636,20 @@ grabArgs(MacroBuf *mb, const MacroEntry *me, const char *se)
 
     addMacro(mb->mc, "0", NULL, b, mb->depth);
     
-    /* Copy args into buf until newline */
+    /* Copy args into buf until lastc */
     *be++ = ' ';
     b = be;	/* Save beginning of args */
     while ((c = *se) != 0) {
 	char *a;
 	se++;
-	if (c == '\n')
+	if (c == lastc)
 		break;
 	if (isblank(c))
 		continue;
 	if (argc > 1)
 		*be++ = ' ';
 	a = be;
-	while (!(isblank(c) || c == '\n')) {
+	while (!(isblank(c) || c == lastc)) {
 		*be++ = c;
 		if ((c = *se) == '\0')
 			break;
@@ -624,10 +659,16 @@ grabArgs(MacroBuf *mb, const MacroEntry *me, const char *se)
 	argc++;
     }
 
-#if 0	/* XXX this is shell syntax, macros need !* to be "the rest" */
+/*
+ * The macro %* analoguous to the shell's $* means "Pass all non-macro
+ * parameters." Consequently, there needs to be a macro that means "Pass all
+ * (including macro parameters) options". This is useful for verifying
+ * parameters during expansion and yet transparently passing all parameters
+ * through for higher level processing (e.g. %description and/or %setup).
+ * This is the (potential) justification for %{**} ...
+ */
     /* Add unexpanded args as macro */
-    addMacro(mb->mc, "*", NULL, b, mb->depth);
-#endif
+    addMacro(mb->mc, "**", NULL, b, mb->depth);
 
 #ifdef NOTYET
     /* XXX if macros can be passed as args ... */
@@ -649,7 +690,7 @@ grabArgs(MacroBuf *mb, const MacroEntry *me, const char *se)
     opts = me->opts;
 
     /* First count number of options ... */
-    optind = 0;
+    saveoptind = optind;	/* XXX optind must be saved on non-linux */
     optc = 0;
     optc++;	/* XXX count argv[0] too */
     while((c = getopt(argc, (char **)argv, opts)) != -1) {
@@ -668,9 +709,9 @@ grabArgs(MacroBuf *mb, const MacroEntry *me, const char *se)
     optv[opte] = NULL;
 
     /* ... and finally define option macros. */
-    optind = 0;
+    optind = saveoptind;	/* XXX optind must be restored on non-linux */
     optc = 0;
-    optc++;	/* XXX count optv[0] */
+    optc++;			/* XXX count optv[0] */
     while((c = getopt(argc, (char **)argv, opts)) != -1) {
 	o = strchr(opts, c);
 	b = be;
@@ -692,7 +733,7 @@ grabArgs(MacroBuf *mb, const MacroEntry *me, const char *se)
 	optc++;
     }
 
-    /* Add macro for each arg. Concatenate args for !* */
+    /* Add macro for each arg. Concatenate args for !*. */
     b = be;
     for (c = optind; c < argc; c++) {
 	sprintf(aname, "%d", (c - optind + 1));
@@ -707,11 +748,11 @@ grabArgs(MacroBuf *mb, const MacroEntry *me, const char *se)
 	optc++;
     }
 
-    /* Add arg count as macro */
-    sprintf(aname, "%d", (argc - optind + 1));
+    /* Add arg count as macro. */
+    sprintf(aname, "%d", (argc - optind));
     addMacro(mb->mc, "#", NULL, aname, mb->depth);
 
-    /* Add unexpanded args as macro */
+    /* Add unexpanded args as macro. */
     addMacro(mb->mc, "*", NULL, b, mb->depth);
 
     return se;
@@ -800,9 +841,6 @@ doFoo(MacroBuf *mb, const char *f, size_t fn, const char *g, size_t glen)
 	} else if (STREQ("F", f, fn)) {
 		b = buf + strlen(buf) + 1;
 		sprintf(b, "file%s.file", buf);
-#if DEAD
-fprintf(stderr, "FILE: \"%s\"\n", b);
-#endif
 	}
 
 	if (b) {
@@ -825,7 +863,7 @@ expandMacro(MacroBuf *mb)
     int c;
     int rc = 0;
     int negate;
-    int grab;
+    char grab;
     int chkexist;
 
     if (++mb->depth > max_macro_depth) {
@@ -857,7 +895,7 @@ expandMacro(MacroBuf *mb)
 	if (mb->depth > 1)	/* XXX full expansion for outermost level */
 		t = mb->t;	/* save expansion pointer for printExpand */
 	negate = 0;
-	grab = 0;
+	grab = '\0';
 	chkexist = 0;
 	switch ((c = *s)) {
 	default:		/* %name substitution */
@@ -876,12 +914,22 @@ expandMacro(MacroBuf *mb)
 			se++;
 		while((c = *se) && (isalnum(c) || c == '_'))
 			se++;
-		if (*se == '*')
+		/* Recognize non-alnum macros too */
+		switch (*se) {
+		case '*':
 			se++;
+			if (*se == '*') se++;
+			break;
+		case '#':
+			se++;
+			break;
+		default:
+			break;
+		}
 		fe = se;
 		/* For "%name " macros ... */
 		if ((c = *fe) && isblank(c))
-			grab = 1;
+			grab = '\n';
 		break;
 	case '(':		/* %(...) shell escape */
 		if ((se = matchchar(s, c, ')')) == NULL) {
@@ -917,11 +965,18 @@ expandMacro(MacroBuf *mb)
 				break;
 			}
 		}
-		for (fe = f; (c = *fe) && !strchr(":}", c);)
+		for (fe = f; (c = *fe) && !strchr(" :}", c);)
 			fe++;
-		if (c == ':') {
+		switch (c) {
+		case ':':
 			g = fe + 1;
 			ge = se - 1;
+			break;
+		case ' ':
+			grab = se[-1];
+			break;
+		default:
+			break;
 		}
 		break;
 	}
@@ -984,7 +1039,7 @@ expandMacro(MacroBuf *mb)
 
 	if (STREQ("dump", f, fn)) {
 		dumpMacroTable(mb->mc, NULL);
-		if (*se == '\n')
+		while (iseol(*se))
 			se++;
 		s = se;
 		continue;
@@ -1065,10 +1120,13 @@ expandMacro(MacroBuf *mb)
 
 	/* Setup args for "%name " macros with opts */
 	if (me && me->opts != NULL) {
-		if (grab)
-			se = grabArgs(mb, me, fe);
-		else {
-		    addMacro(mb->mc, "*", NULL, "", mb->depth);
+		if (grab) {
+			se = grabArgs(mb, me, fe, grab);
+		} else {
+			addMacro(mb->mc, "**", NULL, "", mb->depth);
+			addMacro(mb->mc, "*", NULL, "", mb->depth);
+			addMacro(mb->mc, "#", NULL, "0", mb->depth);
+			addMacro(mb->mc, "0", NULL, me->name, mb->depth);
 		}
 	}
 
@@ -1167,14 +1225,14 @@ addMacro(MacroContext *mc, const char *n, const char *o, const char *b, int leve
 }
 
 void
-delMacro(MacroContext *mc, const char *name)
+delMacro(MacroContext *mc, const char *n)
 {
 	MacroEntry **mep;
 
 	if (mc == NULL)
 		mc = &globalMacroContext;
 	/* If name exists, pop entry */
-	if ((mep = findEntry(mc, name, 0)) != NULL) {
+	if ((mep = findEntry(mc, n, 0)) != NULL) {
 		popMacro(mep);
 		/* If deleted name, sort macro table */
 		if (!(mep && *mep))
@@ -1203,7 +1261,7 @@ initMacros(MacroContext *mc, const char *macrofiles)
 	if (mc == NULL)
 		mc = &globalMacroContext;
 
-	for (mfile = m = strdup(macrofiles); *mfile; mfile = me) {
+	for (mfile = m = xstrdup(macrofiles); *mfile; mfile = me) {
 		FILE *fp;
 		char buf[BUFSIZ];
 
@@ -1324,7 +1382,7 @@ rpmExpand(const char *arg, ...)
     va_list ap;
 
     if (arg == NULL)
-	return strdup("");
+	return xstrdup("");
 
     p = buf;
     strcpy(p, arg);
@@ -1339,7 +1397,7 @@ rpmExpand(const char *arg, ...)
     }
     va_end(ap);
     expandMacros(NULL, NULL, buf, sizeof(buf));
-    return strdup(buf);
+    return xstrdup(buf);	/* XXX build memory leaks */
 }
 
 int
@@ -1378,7 +1436,7 @@ rpmGetPath(const char *path, ...)
     va_list ap;
 
     if (path == NULL)
-	return strdup("");
+	return xstrdup("");
 
     p = buf;
     strcpy(p, path);
@@ -1398,7 +1456,7 @@ rpmGetPath(const char *path, ...)
     }
     va_end(ap);
     expandMacros(NULL, NULL, buf, sizeof(buf));
-    return strdup(buf);
+    return xstrdup(buf);
 }
 
 /* =============================================================== */
@@ -1462,8 +1520,7 @@ main(int argc, char *argv[])
 	dumpMacroTable(NULL, NULL);
 
 	if ((fp = fopen(testfile, "r")) != NULL) {
-		while(fgets(buf, sizeof(buf), fp)) {
-			buf[strlen(buf)-1] = '\0';
+		while(rdcl(buf, sizeof(buf), fp, 1)) {
 			x = expandMacros(NULL, NULL, buf, sizeof(buf));
 			fprintf(stderr, "%d->%s\n", x, buf);
 			memset(buf, 0, sizeof(buf));
@@ -1471,8 +1528,7 @@ main(int argc, char *argv[])
 		fclose(fp);
 	}
 
-	while(fgets(buf, sizeof(buf), stdin)) {
-		buf[strlen(buf)-1] = '\0';
+	while(rdcl(buf, sizeof(buf), stdin, 1)) {
 		x = expandMacros(NULL, NULL, buf, sizeof(buf));
 		fprintf(stderr, "%d->%s\n <-\n", x, buf);
 		memset(buf, 0, sizeof(buf));
