@@ -1,11 +1,12 @@
 #include "system.h"
 
-#include "rpmlib.h"
-#include "rpmmacro.h"	/* XXX for rpmGetPath */
+#include <rpmlib.h>
+#include <rpmmacro.h>	/* XXX for rpmGetPath */
 
 #include "rpmdb.h"
 
-int rpmdbRebuild(const char * rootdir) {
+int rpmdbRebuild(const char * rootdir)
+{
     rpmdb olddb, newdb;
     const char * dbpath = NULL;
     const char * newdbpath = NULL;
