@@ -4,14 +4,29 @@
 #include <signal.h>
 #include <sys/signal.h>
 
-#include "rpmlib.h"
+#include <rpmlib.h>
+#include <rpmmacro.h>	/* XXX for rpmGetPath */
 
 #include "falloc.h"
 #include "fprint.h"
 #include "misc.h"
-#include "rpmmacro.h"	/* XXX for rpmGetPath */
 #include "rpmdb.h"
 
+/*@access dbiIndexSet@*/
+/*@access dbiIndexRecord@*/
+
+const char *rpmdb_filenames[] = {
+    "packages.rpm",
+    "nameindex.rpm",
+    "fileindex.rpm",
+    "groupindex.rpm",
+    "requiredby.rpm",
+    "providesindex.rpm",
+    "conflictsindex.rpm",
+    "triggerindex.rpm",
+    NULL
+};
+
 /* XXX the signal handling in here is not thread safe */
 
 /* the requiredbyIndex isn't stricly necessary. In a perfect world, we could
@@ -29,35 +44,25 @@ struct rpmdb_s {
     dbiIndex * requiredbyIndex, * conflictsIndex, * triggerIndex;
 };
 
-struct intMatch {
-    dbiIndexRecord rec;
-    int fpNum;
-};
-
-static void removeIndexEntry(dbiIndex * dbi, char * name, dbiIndexRecord rec,
-		             int tolerant, char * idxName);
-static int addIndexEntry(dbiIndex * idx, char * index, unsigned int offset,
-		         unsigned int fileNumber);
-static void blockSignals(void);
-static void unblockSignals(void);
-static int intMatchCmp(const void * one, const void * two);
-
 static sigset_t signalMask;
 
-static int intMatchCmp(const void * one, const void * two) {
-    const struct intMatch * a = one;
-    const struct intMatch * b = two;
+static void blockSignals(void)
+{
+    sigset_t newMask;
 
-    if (a->rec.recOffset < b->rec.recOffset)
-	return -1;
-    else if (a->rec.recOffset > b->rec.recOffset)
-	return 1;
-
-    return 0;
+    sigfillset(&newMask);		/* block all signals */
+    sigprocmask(SIG_BLOCK, &newMask, &signalMask);
 }
 
-int rpmdbOpenForTraversal(const char * prefix, rpmdb * rpmdbp) {
+static void unblockSignals(void)
+{
+    sigprocmask(SIG_SETMASK, &signalMask, NULL);
+}
+
+int rpmdbOpenForTraversal(const char * prefix, rpmdb * rpmdbp)
+{
     const char * dbpath;
+    int rc = 0;
 
     dbpath = rpmGetPath("%{_dbpath}", NULL);
     if (dbpath == NULL || dbpath[0] == '%') {
@@ -67,13 +72,14 @@ int rpmdbOpenForTraversal(const char * prefix, rpmdb * rpmdbp) {
 
     if (openDatabase(prefix, dbpath, rpmdbp, O_RDONLY, 0644, 
 		     RPMDB_FLAG_MINIMAL)) {
-	return 1;
+	rc = 1;
     }
     xfree(dbpath);
-    return 0;
+    return rc;
 }
 
-int rpmdbOpen (const char * prefix, rpmdb *rpmdbp, int mode, int perms) {
+int rpmdbOpen (const char * prefix, rpmdb *rpmdbp, int mode, int perms)
+{
     const char * dbpath;
     int rc;
 
@@ -88,7 +94,8 @@ int rpmdbOpen (const char * prefix, rpmdb *rpmdbp, int mode, int perms) {
     return rc;
 }
 
-int rpmdbInit (const char * prefix, int perms) {
+int rpmdbInit (const char * prefix, int perms)
+{
     rpmdb db;
     const char * dbpath;
     int rc;
@@ -106,18 +113,18 @@ int rpmdbInit (const char * prefix, int perms) {
 }
 
 static int openDbFile(const char * prefix, const char * dbpath, const char * shortName, 
-		      int justCheck, int perms, dbiIndex ** db, DBTYPE type){
-    char * filename = alloca(strlen(prefix) + strlen(dbpath) + 
-			     strlen(shortName) + 20);
+	 int justCheck, int mode, int perms, dbiIndex ** db, DBTYPE type)
+{
+    int len = (prefix ? strlen(prefix) : 0) + strlen(dbpath) + strlen(shortName) + 1;
+    char * filename = alloca(len);
 
-    if (!prefix) prefix="";
-
-    strcpy(filename, prefix); 
+    *filename = '\0';
+    if (prefix && *prefix) strcat(filename, prefix); 
     strcat(filename, dbpath);
     strcat(filename, shortName);
 
     if (!justCheck || !rpmfileexists(filename)) {
-	*db = dbiOpenIndex(filename, perms, 0644, type);
+	*db = dbiOpenIndex(filename, mode, perms, type);
 	if (!*db) {
 	    return 1;
 	}
@@ -126,10 +133,25 @@ static int openDbFile(const char * prefix, const char * dbpath, const char * sho
     return 0;
 }
 
+static /*@only@*/ rpmdb newRpmdb(void)
+{
+    rpmdb db = xmalloc(sizeof(*db));
+    db->pkgs = NULL;
+    db->nameIndex = NULL;
+    db->fileIndex = NULL;
+    db->groupIndex = NULL;
+    db->providesIndex = NULL;
+    db->requiredbyIndex = NULL;
+    db->conflictsIndex = NULL;
+    db->triggerIndex = NULL;
+    return db;
+}
+
 int openDatabase(const char * prefix, const char * dbpath, rpmdb *rpmdbp, int mode, 
-		 int perms, int flags) {
+		 int perms, int flags)
+{
     char * filename;
-    struct rpmdb_s db;
+    rpmdb db;
     int i, rc;
     struct flock lockinfo;
     int justcheck = flags & RPMDB_FLAG_JUSTCHECK;
@@ -139,6 +161,9 @@ int openDatabase(const char * prefix, const char * dbpath, rpmdb *rpmdbp, int mo
     if (mode & O_WRONLY) 
 	return 1;
 
+    if (!(perms & 0600))	/* XXX sanity */
+	perms = 0644;
+
     /* we should accept NULL as a valid prefix */
     if (!prefix) prefix="";
 
@@ -161,11 +186,10 @@ int openDatabase(const char * prefix, const char * dbpath, rpmdb *rpmdbp, int mo
 
     strcat(filename, "packages.rpm");
 
-    memset(&db, 0, sizeof(db));
+    db = newRpmdb();
 
     if (!justcheck || !rpmfileexists(filename)) {
-	db.pkgs = faOpen(filename, mode, 0644);
-	if (db.pkgs == NULL) {
+	if ((db->pkgs = faOpen(filename, mode, perms)) == NULL) {
 	    rpmError(RPMERR_DBOPEN, _("failed to open %s\n"), filename);
 	    return 1;
 	}
@@ -178,37 +202,46 @@ int openDatabase(const char * prefix, const char * dbpath, rpmdb *rpmdbp, int mo
 	
 	if (mode & O_RDWR) {
 	    lockinfo.l_type = F_WRLCK;
-	    if (faFcntl(db.pkgs, F_SETLK, (void *) &lockinfo)) {
+	    if (faFcntl(db->pkgs, F_SETLK, (void *) &lockinfo)) {
 		rpmError(RPMERR_FLOCK, _("cannot get %s lock on database"), 
 			 _("exclusive"));
+		rpmdbClose(db);
 		return 1;
 	    } 
 	} else {
 	    lockinfo.l_type = F_RDLCK;
-	    if (faFcntl(db.pkgs, F_SETLK, (void *) &lockinfo)) {
+	    if (faFcntl(db->pkgs, F_SETLK, (void *) &lockinfo)) {
 		rpmError(RPMERR_FLOCK, _("cannot get %s lock on database"), 
 			 _("shared"));
+		rpmdbClose(db);
 		return 1;
 	    } 
 	}
     }
 
-    rc = openDbFile(prefix, dbpath, "nameindex.rpm", justcheck, mode,
-		    &db.nameIndex, DB_HASH);
+    rc = openDbFile(prefix, dbpath, "nameindex.rpm", justcheck, mode, perms,
+		    &db->nameIndex, DB_HASH);
 
     if (minimal) {
-	*rpmdbp = malloc(sizeof(struct rpmdb_s));
-	**rpmdbp = db;
+	*rpmdbp = xmalloc(sizeof(struct rpmdb_s));
+	if (rpmdbp)
+	    *rpmdbp = db;	/* structure assignment */
+	else
+	    rpmdbClose(db);
 	return 0;
     }
 
     if (!rc)
-	rc = openDbFile(prefix, dbpath, "fileindex.rpm", justcheck, mode,
-			&db.fileIndex, DB_HASH);
+	rc = openDbFile(prefix, dbpath, "fileindex.rpm", justcheck, mode, perms,
+			&db->fileIndex, DB_HASH);
 
     /* We used to store the fileindexes as complete paths, rather then
        plain basenames. Let's see which version we are... */
-    if (!justcheck && !dbiGetFirstKey(db.fileIndex, &akey)) {
+    /*
+     * XXX FIXME: db->fileindex can be NULL under pathological (e.g. mixed
+     * XXX db1/db2 linkage) conditions.
+     */
+    if (!justcheck && !dbiGetFirstKey(db->fileIndex, &akey)) {
 	if (strchr(akey, '/')) {
 	    rpmError(RPMERR_OLDDB, _("old format database is present; "
 			"use --rebuilddb to generate a new format database"));
@@ -218,44 +251,31 @@ int openDatabase(const char * prefix, const char * dbpath, rpmdb *rpmdbp, int mo
     }
 
     if (!rc)
-	rc = openDbFile(prefix, dbpath, "providesindex.rpm", justcheck, mode,
-			&db.providesIndex, DB_HASH);
+	rc = openDbFile(prefix, dbpath, "providesindex.rpm", justcheck, mode, perms,
+			&db->providesIndex, DB_HASH);
     if (!rc)
-	rc = openDbFile(prefix, dbpath, "requiredby.rpm", justcheck, mode,
-			&db.requiredbyIndex, DB_HASH);
+	rc = openDbFile(prefix, dbpath, "requiredby.rpm", justcheck, mode, perms,
+			&db->requiredbyIndex, DB_HASH);
     if (!rc)
-	rc = openDbFile(prefix, dbpath, "conflictsindex.rpm", justcheck, mode,
-			&db.conflictsIndex, DB_HASH);
+	rc = openDbFile(prefix, dbpath, "conflictsindex.rpm", justcheck, mode, perms,
+			&db->conflictsIndex, DB_HASH);
     if (!rc)
-	rc = openDbFile(prefix, dbpath, "groupindex.rpm", justcheck, mode,
-			&db.groupIndex, DB_HASH);
+	rc = openDbFile(prefix, dbpath, "groupindex.rpm", justcheck, mode, perms,
+			&db->groupIndex, DB_HASH);
     if (!rc)
-	rc = openDbFile(prefix, dbpath, "triggerindex.rpm", justcheck, mode,
-			&db.triggerIndex, DB_HASH);
+	rc = openDbFile(prefix, dbpath, "triggerindex.rpm", justcheck, mode, perms,
+			&db->triggerIndex, DB_HASH);
 
-    if (rc) {
-	faClose(db.pkgs);
-	if (db.nameIndex) dbiCloseIndex(db.nameIndex);
-	if (db.fileIndex) dbiCloseIndex(db.fileIndex);
-	if (db.providesIndex) dbiCloseIndex(db.providesIndex);
-	if (db.requiredbyIndex) dbiCloseIndex(db.requiredbyIndex);
-	if (db.conflictsIndex) dbiCloseIndex(db.conflictsIndex);
-	if (db.groupIndex) dbiCloseIndex(db.groupIndex);
-	if (db.triggerIndex) dbiCloseIndex(db.triggerIndex);
-	return 1;
-    }
+    if (rc || justcheck || rpmdbp == NULL)
+	rpmdbClose(db);
+     else
+	*rpmdbp = db;
 
-    *rpmdbp = malloc(sizeof(struct rpmdb_s));
-    **rpmdbp = db;
-
-    if (justcheck) {
-	rpmdbClose(*rpmdbp);
-    }
-
-    return 0;
+     return rc;
 }
 
-void rpmdbClose (rpmdb db) {
+void rpmdbClose (rpmdb db)
+{
     if (db->pkgs != NULL) faClose(db->pkgs);
     if (db->fileIndex) dbiCloseIndex(db->fileIndex);
     if (db->groupIndex) dbiCloseIndex(db->groupIndex);
@@ -276,13 +296,59 @@ int rpmdbNextRecNum(rpmdb db, unsigned int lastOffset) {
     return faNextOffset(db->pkgs, lastOffset);
 }
 
-Header rpmdbGetRecord(rpmdb db, unsigned int offset) {
+static Header doGetRecord(rpmdb db, unsigned int offset, int pristine) {
+    Header h;
+    char ** fileList;
+    char ** newList;
+    int fileCount = 0;
+    int i;
+
     (void)faLseek(db->pkgs, offset, SEEK_SET);
 
-    return headerRead(faFileno(db->pkgs), HEADER_MAGIC_NO);
+    h = headerRead(faFileno(db->pkgs), HEADER_MAGIC_NO);
+
+    if (pristine) return h;
+
+    /* the RPM used to buildmuch of RH 5.1 could produce packages whose
+       file lists did not have leading /'s. Now is a good time to fix
+       that */
+
+    if (!headerGetEntryMinMemory(h, RPMTAG_FILENAMES, NULL, 
+			   (void **) &fileList, &fileCount)) return h;
+
+    for (i = 0; i < fileCount; i++) 
+	if (*fileList[i] != '/') break;
+
+    if (i == fileCount) {
+	free(fileList);
+	return h;
+    }
+
+    /* bad header -- let's clean it up */
+    newList = alloca(sizeof(*newList) * fileCount);
+    for (i = 0; i < fileCount; i++) {
+	newList[i] = alloca(strlen(fileList[i]) + 2);
+	if (*fileList[i] == '/')
+	    strcpy(newList[i], fileList[i]);
+	else
+	    sprintf(newList[i], "/%s", fileList[i]);
+    }
+
+    free(fileList);
+
+    headerModifyEntry(h, RPMTAG_FILENAMES, RPM_STRING_ARRAY_TYPE, 
+		      newList, fileCount);
+
+    return h;
 }
 
-int rpmdbFindByFile(rpmdb db, const char * filespec, dbiIndexSet * matches) {
+Header rpmdbGetRecord(rpmdb db, unsigned int offset)
+{
+    return doGetRecord(db, offset, 0);
+}
+
+int rpmdbFindByFile(rpmdb db, const char * filespec, dbiIndexSet * matches)
+{
     const char * basename;
     fingerPrint fp1, fp2;
     dbiIndexSet allMatches;
@@ -304,7 +370,7 @@ int rpmdbFindByFile(rpmdb db, const char * filespec, dbiIndexSet * matches) {
     *matches = dbiCreateIndexRecord();
     i = 0;
     while (i < allMatches.count) {
-	if (!(h = rpmdbGetRecord(db, allMatches.recs[i].recOffset))) {
+	if ((h = rpmdbGetRecord(db, allMatches.recs[i].recOffset)) == NULL) {
 	    i++;
 	    continue;
 	}
@@ -361,7 +427,8 @@ int rpmdbFindPackage(rpmdb db, const char * name, dbiIndexSet * matches) {
 }
 
 static void removeIndexEntry(dbiIndex * dbi, char * key, dbiIndexRecord rec,
-		             int tolerant, char * idxName) {
+		             int tolerant, char * idxName)
+{
     int rc;
     dbiIndexSet matches;
     
@@ -388,7 +455,8 @@ static void removeIndexEntry(dbiIndex * dbi, char * key, dbiIndexRecord rec,
     }
 }
 
-int rpmdbRemove(rpmdb db, unsigned int offset, int tolerant) {
+int rpmdbRemove(rpmdb db, unsigned int offset, int tolerant)
+{
     Header h;
     char * name, * group;
     int type;
@@ -425,7 +493,7 @@ int rpmdbRemove(rpmdb db, unsigned int offset, int tolerant) {
 	removeIndexEntry(db->groupIndex, group, rec, tolerant, "group index");
     }
 
-    if (headerGetEntry(h, RPMTAG_PROVIDES, &type, (void **) &providesList, 
+    if (headerGetEntry(h, RPMTAG_PROVIDENAME, &type, (void **) &providesList, 
 	 &count)) {
 	for (i = 0; i < count; i++) {
 	    rpmMessage(RPMMESS_DEBUG, _("removing provides index for %s\n"), 
@@ -508,8 +576,9 @@ int rpmdbRemove(rpmdb db, unsigned int offset, int tolerant) {
     return 0;
 }
 
-static int addIndexEntry(dbiIndex * idx, char * index, unsigned int offset,
-		         unsigned int fileNumber) {
+static int addIndexEntry(dbiIndex *idx, const char *index, unsigned int offset,
+		         unsigned int fileNumber)
+{
     dbiIndexSet set;
     dbiIndexRecord irec;   
     int rc;
@@ -524,25 +593,26 @@ static int addIndexEntry(dbiIndex * idx, char * index, unsigned int offset,
 	set = dbiCreateIndexRecord();
     dbiAppendIndexRecord(&set, irec);
     if (dbiUpdateIndex(idx, index, &set))
-	exit(1);
+	exit(EXIT_FAILURE);
     dbiFreeIndexRecord(set);
     return 0;
 }
 
-int rpmdbAdd(rpmdb db, Header dbentry) {
+int rpmdbAdd(rpmdb db, Header dbentry)
+{
     unsigned int dboffset;
     unsigned int i, j;
-    char ** fileList;
-    char ** providesList;
-    char ** requiredbyList;
-    char ** conflictList;
-    char ** triggerList;
-    char * name, * group;
+    const char ** fileList;
+    const char ** providesList;
+    const char ** requiredbyList;
+    const char ** conflictList;
+    const char ** triggerList;
+    const char * name;
+    const char * group;
     int count = 0, providesCount = 0, requiredbyCount = 0, conflictCount = 0;
     int triggerCount = 0;
     int type;
     int rc = 0;
-    char * basename;
 
     headerGetEntry(dbentry, RPMTAG_NAME, &type, (void **) &name, &count);
     headerGetEntry(dbentry, RPMTAG_GROUP, &type, (void **) &group, &count);
@@ -552,7 +622,7 @@ int rpmdbAdd(rpmdb db, Header dbentry) {
     count = 0;
     headerGetEntry(dbentry, RPMTAG_FILENAMES, &type, (void **) &fileList, 
 	           &count);
-    headerGetEntry(dbentry, RPMTAG_PROVIDES, &type, (void **) &providesList, 
+    headerGetEntry(dbentry, RPMTAG_PROVIDENAME, &type, (void **) &providesList, 
 	           &providesCount);
     headerGetEntry(dbentry, RPMTAG_REQUIRENAME, &type, 
 		   (void **) &requiredbyList, &requiredbyCount);
@@ -565,18 +635,16 @@ int rpmdbAdd(rpmdb db, Header dbentry) {
 
     dboffset = faAlloc(db->pkgs, headerSizeof(dbentry, HEADER_MAGIC_NO));
     if (!dboffset) {
-	rpmError(RPMERR_DBCORRUPT, _("cannot allocate space for database"));
-	unblockSignals();
-	if (providesCount) free(providesList);
-	if (requiredbyCount) free(requiredbyList);
-	if (conflictCount) free(conflictList);
-	if (triggerCount) free(triggerList);
-	if (count) free(fileList);
-	return 1;
+	rc = 1;
+    } else {
+	(void)faLseek(db->pkgs, dboffset, SEEK_SET);
+	rc = headerWrite(faFileno(db->pkgs), dbentry, HEADER_MAGIC_NO);
     }
-    (void)faLseek(db->pkgs, dboffset, SEEK_SET);
 
-    headerWrite(faFileno(db->pkgs), dbentry, HEADER_MAGIC_NO);
+    if (rc) {
+	rpmError(RPMERR_DBCORRUPT, _("cannot allocate space for database"));
+	goto exit;
+    }
 
     /* Now update the appropriate indexes */
     if (addIndexEntry(db->nameIndex, name, dboffset, 0))
@@ -603,6 +671,7 @@ int rpmdbAdd(rpmdb db, Header dbentry) {
 	rc += addIndexEntry(db->providesIndex, providesList[i], dboffset, 0);
 
     for (i = 0; i < count; i++) {
+	const char * basename;
 	basename = strrchr(fileList[i], '/');
 	if (!basename) 
 	    basename = fileList[i];
@@ -619,6 +688,7 @@ int rpmdbAdd(rpmdb db, Header dbentry) {
     dbiSyncIndex(db->requiredbyIndex);
     dbiSyncIndex(db->triggerIndex);
 
+exit:
     unblockSignals();
 
     if (requiredbyCount) free(requiredbyList);
@@ -630,11 +700,13 @@ int rpmdbAdd(rpmdb db, Header dbentry) {
     return rc;
 }
 
-int rpmdbUpdateRecord(rpmdb db, int offset, Header newHeader) {
+int rpmdbUpdateRecord(rpmdb db, int offset, Header newHeader)
+{
     Header oldHeader;
     int oldSize;
+    int rc = 0;
 
-    oldHeader = rpmdbGetRecord(db, offset);
+    oldHeader = doGetRecord(db, offset, 1);
     if (oldHeader == NULL) {
 	rpmError(RPMERR_DBCORRUPT, _("cannot read header at %d for update"),
 		offset);
@@ -656,27 +728,18 @@ int rpmdbUpdateRecord(rpmdb db, int offset, Header newHeader) {
 
 	(void)faLseek(db->pkgs, offset, SEEK_SET);
 
-	headerWrite(faFileno(db->pkgs), newHeader, HEADER_MAGIC_NO);
+	rc = headerWrite(faFileno(db->pkgs), newHeader, HEADER_MAGIC_NO);
 
 	unblockSignals();
     }
 
-    return 0;
+    return rc;
 }
 
-static void blockSignals(void) {
-    sigset_t newMask;
-
-    sigfillset(&newMask);		/* block all signals */
-    sigprocmask(SIG_BLOCK, &newMask, &signalMask);
-}
-
-static void unblockSignals(void) {
-    sigprocmask(SIG_SETMASK, &signalMask, NULL);
-}
-
-void rpmdbRemoveDatabase(const char * rootdir, const char * dbpath) { 
+void rpmdbRemoveDatabase(const char * rootdir, const char * dbpath)
+{ 
     int i;
+    const char **rpmdbfnp;
     char * filename;
 
     i = strlen(dbpath);
@@ -690,30 +753,20 @@ void rpmdbRemoveDatabase(const char * rootdir, const char * dbpath) {
     
     filename = alloca(strlen(rootdir) + strlen(dbpath) + 40);
 
-    sprintf(filename, "%s/%s/packages.rpm", rootdir, dbpath);
-    unlink(filename);
+    for (rpmdbfnp = rpmdb_filenames; *rpmdbfnp; rpmdbfnp++) {
+	sprintf(filename, "%s/%s/%s", rootdir, dbpath, *rpmdbfnp);
+	unlink(filename);
+    }
 
-    sprintf(filename, "%s/%s/nameindex.rpm", rootdir, dbpath);
-    unlink(filename);
+    sprintf(filename, "%s/%s", rootdir, dbpath);
+    rmdir(filename);
 
-    sprintf(filename, "%s/%s/fileindex.rpm", rootdir, dbpath);
-    unlink(filename);
-
-    sprintf(filename, "%s/%s/groupindex.rpm", rootdir, dbpath);
-    unlink(filename);
-
-    sprintf(filename, "%s/%s/requiredby.rpm", rootdir, dbpath);
-    unlink(filename);
-
-    sprintf(filename, "%s/%s/providesindex.rpm", rootdir, dbpath);
-    unlink(filename);
-
-    sprintf(filename, "%s/%s/conflictsindex.rpm", rootdir, dbpath);
-    unlink(filename);
 }
 
-int rpmdbMoveDatabase(const char * rootdir, const char * olddbpath, const char * newdbpath) {
+int rpmdbMoveDatabase(const char * rootdir, const char * olddbpath, const char * newdbpath)
+{
     int i;
+    const char **rpmdbfnp;
     char * ofilename, * nfilename;
     int rc = 0;
  
@@ -738,81 +791,74 @@ int rpmdbMoveDatabase(const char * rootdir, const char * olddbpath, const char *
     ofilename = alloca(strlen(rootdir) + strlen(olddbpath) + 40);
     nfilename = alloca(strlen(rootdir) + strlen(newdbpath) + 40);
 
-    sprintf(ofilename, "%s/%s/packages.rpm", rootdir, olddbpath);
-    sprintf(nfilename, "%s/%s/packages.rpm", rootdir, newdbpath);
-    if (rename(ofilename, nfilename)) rc = 1;
-
-    sprintf(ofilename, "%s/%s/nameindex.rpm", rootdir, olddbpath);
-    sprintf(nfilename, "%s/%s/nameindex.rpm", rootdir, newdbpath);
-    if (rename(ofilename, nfilename)) rc = 1;
-
-    sprintf(ofilename, "%s/%s/fileindex.rpm", rootdir, olddbpath);
-    sprintf(nfilename, "%s/%s/fileindex.rpm", rootdir, newdbpath);
-    if (rename(ofilename, nfilename)) rc = 1;
-
-    sprintf(ofilename, "%s/%s/groupindex.rpm", rootdir, olddbpath);
-    sprintf(nfilename, "%s/%s/groupindex.rpm", rootdir, newdbpath);
-    if (rename(ofilename, nfilename)) rc = 1;
-
-    sprintf(ofilename, "%s/%s/requiredby.rpm", rootdir, olddbpath);
-    sprintf(nfilename, "%s/%s/requiredby.rpm", rootdir, newdbpath);
-    if (rename(ofilename, nfilename)) rc = 1;
-
-    sprintf(ofilename, "%s/%s/providesindex.rpm", rootdir, olddbpath);
-    sprintf(nfilename, "%s/%s/providesindex.rpm", rootdir, newdbpath);
-    if (rename(ofilename, nfilename)) rc = 1;
-
-    sprintf(ofilename, "%s/%s/conflictsindex.rpm", rootdir, olddbpath);
-    sprintf(nfilename, "%s/%s/conflictsindex.rpm", rootdir, newdbpath);
-    if (rename(ofilename, nfilename)) rc = 1;
-
-    sprintf(ofilename, "%s/%s/triggerindex.rpm", rootdir, olddbpath);
-    sprintf(nfilename, "%s/%s/triggerindex.rpm", rootdir, newdbpath);
-    if (rename(ofilename, nfilename)) rc = 1;
+    for (rpmdbfnp = rpmdb_filenames; *rpmdbfnp; rpmdbfnp++) {
+	sprintf(ofilename, "%s/%s/%s", rootdir, olddbpath, *rpmdbfnp);
+	sprintf(nfilename, "%s/%s/%s", rootdir, newdbpath, *rpmdbfnp);
+	if (rename(ofilename, nfilename)) rc = 1;
+    }
 
     return rc;
 }
 
+struct intMatch {
+    dbiIndexRecord rec;
+    int fpNum;
+};
+
+static int intMatchCmp(const void * one, const void * two)
+{
+    const struct intMatch * a = one;
+    const struct intMatch * b = two;
+
+    if (a->rec.recOffset < b->rec.recOffset)
+	return -1;
+    else if (a->rec.recOffset > b->rec.recOffset)
+	return 1;
+
+    return 0;
+}
+
 int rpmdbFindFpList(rpmdb db, fingerPrint * fpList, dbiIndexSet * matchList, 
-		    int numItems) {
-    struct intMatch * intMatches, * im;
+		    int numItems)
+{
+    int numIntMatches = 0;
+    int intMatchesAlloced = numItems;
+    struct intMatch * intMatches;
     int i, j;
-    dbiIndexSet matches;
-    int intMatchesAlloced, numIntMatches;
     int start, end;
-    int num, rc;
-    const char ** fullfl, **fl;
-    int_32 fc;
-    fingerPrint * fps;
-    Header h;
+    int num;
 
     /* this may be worth batching by basename, but probably not as
        basenames are quite unique as it is */
 
-    intMatchesAlloced = numItems;
-    numIntMatches = 0;
-    im = intMatches = malloc(sizeof(*intMatches) * intMatchesAlloced);
+    intMatches = xcalloc(intMatchesAlloced, sizeof(*intMatches));
 
-    for (i = 0; i < numItems; i++, im++) {
-        rc = dbiSearchIndex(db->fileIndex, fpList[i].basename, &matches);
-	if (rc == 2) {
-	    im = intMatches;
+    /* Gather all matches from the database */
+    for (i = 0; i < numItems; i++) {
+	dbiIndexSet matches;
+	switch (dbiSearchIndex(db->fileIndex, fpList[i].basename, &matches)) {
+	default:
+	    break;
+	case 2:
 	    free(intMatches);
 	    return 1;
-	} else if (!rc) {
-	    if ((numIntMatches + matches.count) >= intMatchesAlloced) {
-		intMatchesAlloced += matches.count;
+	    /*@notreached@*/ break;
+	case 0:
+	    if ((numIntMatches + dbiIndexSetCount(matches)) >= intMatchesAlloced) {
+		intMatchesAlloced += dbiIndexSetCount(matches);
 		intMatchesAlloced += intMatchesAlloced / 5;
-		intMatches = realloc(intMatches, 
+		intMatches = xrealloc(intMatches, 
 				     sizeof(*intMatches) * intMatchesAlloced);
 	    }
 
-	    for (j = 0; j < matches.count; j++) {
+	    for (j = 0; j < dbiIndexSetCount(matches); j++) {
+		/* structure assignment */
 		intMatches[numIntMatches].rec = matches.recs[j];
 		intMatches[numIntMatches++].fpNum = i;
 	    }
 
 	    dbiFreeIndexRecord(matches);
+	    break;
 	}
     }
 
@@ -822,32 +868,44 @@ int rpmdbFindFpList(rpmdb db, fingerPrint * fpList, dbiIndexSet * matchList,
     for (i = 0; i < numItems; i++)
 	matchList[i] = dbiCreateIndexRecord();
 
-    start = 0;
-    while (start < numIntMatches) {
-	im = intMatches + start;
-	end = start + 1;
-	while (end < numIntMatches && 
-	    (im->rec.recOffset == intMatches[end].rec.recOffset))
-	    end++;
+    /* For each set of files matched in a package ... */
+    for (start = 0; start < numIntMatches; start = end) {
+	struct intMatch * im;
+	Header h;
+	fingerPrint * fps;
 
+	im = intMatches + start;
+
+	/* Find the end of the set of matched files in this package. */
+	for (end = start + 1; end < numIntMatches; end++) {
+	    if (im->rec.recOffset != intMatches[end].rec.recOffset)
+		break;
+	}
 	num = end - start;
+
+	/* Compute fingerprints for each file match in this package. */
 	h = rpmdbGetRecord(db, im->rec.recOffset);
-	if (!h) {
+	if (h == NULL) {
 	    free(intMatches);
 	    return 1;
 	}
 
-	headerGetEntryMinMemory(h, RPMTAG_FILENAMES, NULL, 
+	{   const char ** fullfl, **fl;
+	    int_32 fc;
+	   
+	    headerGetEntryMinMemory(h, RPMTAG_FILENAMES, NULL, 
 				(void **) &fullfl, &fc);
 
-	fl = malloc(sizeof(*fl) * num);
-	for (i = 0; i < num; i++)
-	    fl[i] = fullfl[im[i].rec.fileNumber];
-	free(fullfl);
-	fps = malloc(sizeof(*fps) * num);
-	fpLookupList(fl, fps, num, 1);
-	free(fl);
+	    fl = xcalloc(num, sizeof(*fl));
+	    for (i = 0; i < num; i++)
+		fl[i] = fullfl[im[i].rec.fileNumber];
+	    free(fullfl);
+	    fps = xcalloc(num, sizeof(*fps));
+	    fpLookupList(fl, fps, num, 1);
+	    free(fl);
+	}
 
+	/* Add db (recnum,filenum) to list for fingerprint matches. */
 	for (i = 0; i < num; i++) {
 	    j = im[i].fpNum;
 	    if (FP_EQUAL(fps[i], fpList[j]))
@@ -858,7 +916,6 @@ int rpmdbFindFpList(rpmdb db, fingerPrint * fpList, dbiIndexSet * matchList,
 
 	free(fps);
 
-	start = end;
     }
 
     free(intMatches);
