@@ -1,7 +1,8 @@
-#include <stdarg.h>
-#include <stdio.h>
+#include "system.h"
 
-#include "rpmlib.h"
+#include <stdarg.h>
+
+#include <rpmlib.h>
 
 static struct err {
     int code;
