@@ -13,124 +13,112 @@
 #include <asm/byteorder.h>
 #endif
 
-#include "rpmlib.h"
-#include "rpmmacro.h"	/* XXX for rpmGetPath */
+#include <rpmlib.h>
+#include <rpmmacro.h>	/* XXX for rpmGetPath */
 
 #include "md5.h"
 #include "misc.h"
 #include "rpmlead.h"
 #include "signature.h"
 
-typedef int (*md5func)(const char * fn, unsigned char * digest);
-
-static int makePGPSignature(const char *file, void **sig, int_32 *size,
-			    const char *passPhrase, int sigTag);
-static int makeGPGSignature(const char *file, void **sig, int_32 *size,
-			    const char *passPhrase);
-static int checkSize(FD_t fd, int size, int sigsize);
-static int verifySizeSignature(const char *datafile, int_32 size, char *result);
-static int verifyMD5Signature(const char *datafile, unsigned char *sig,
-			      char *result, md5func fn);
-static int verifyPGPSignature(const char *datafile, void *sig,
-			      int count, char *result, int sigTag);
-static int verifyGPGSignature(const char *datafile, void *sig,
-			      int count, char *result);
-static int checkPassPhrase(const char *passPhrase, const int sigTag);
+typedef int (*md5func)(const char * fn, /*@out@*/unsigned char * digest);
 
 int rpmLookupSignatureType(int action)
 {
-    static int rc = 0;
+    static int disabled = 0;
+    int rc = 0;
 
-    switch (action)
-    {
-	case RPMLOOKUPSIG_DISABLE:
-	    rc = -2;
-	    return 0;
-        case RPMLOOKUPSIG_ENABLE:
+    switch (action) {
+    case RPMLOOKUPSIG_DISABLE:
+	disabled = -2;
+	break;
+    case RPMLOOKUPSIG_ENABLE:
+	disabled = 0;
+	/* fall through */
+	/*@fallthrough@*/
+    case RPMLOOKUPSIG_QUERY:
+	if (disabled)
+	    break;	/* Disabled */
+      { const char *name = rpmExpand("%{_signature}", NULL);
+	if (!(name && *name != '%'))
 	    rc = 0;
-	    /* fall through */
-        case RPMLOOKUPSIG_QUERY:
-	    if (rc == -2)
-	        return 0;	/* Disabled */
-            else
-	    {
-		const char *name = rpmExpand("%{_signature}", NULL);
-		if (!(name && *name != '%'))
-		    rc = 0;
-		else if (!strcasecmp(name, "none"))
-		    rc = 0;
-		else if (!strcasecmp(name, "pgp"))
-		    rc = RPMSIGTAG_PGP;
-		else if (!strcasecmp(name, "pgp5"))
-		    rc = RPMSIGTAG_PGP5;
-		else if (!strcasecmp(name, "gpg"))
-		    rc = RPMSIGTAG_GPG;
-		else
-		    rc = -1;	/* Invalid %_signature spec in macro file */
-		xfree(name);
-	    }
-	    break;
+	else if (!strcasecmp(name, "none"))
+	    rc = 0;
+	else if (!strcasecmp(name, "pgp"))
+	    rc = RPMSIGTAG_PGP;
+	else if (!strcasecmp(name, "pgp5"))	/* XXX legacy */
+	    rc = RPMSIGTAG_PGP;
+	else if (!strcasecmp(name, "gpg"))
+	    rc = RPMSIGTAG_GPG;
+	else
+	    rc = -1;	/* Invalid %_signature spec in macro file */
+	xfree(name);
+      }	break;
     }
-
     return rc;
 }
 
 /* rpmDetectPGPVersion() returns the absolute path to the "pgp"  */
 /* executable of the requested version, or NULL when none found. */
 
-const char * rpmDetectPGPVersion(int sigTag)
+const char * rpmDetectPGPVersion(pgpVersion *pgpVer)
 {
     /* Actually this should support having more then one pgp version. */ 
     /* At the moment only one version is possible since we only       */
     /* have one %_pgpbin and one %_pgp_path.                          */
 
-    static int pgp_version;
+    static pgpVersion saved_pgp_version = PGP_UNKNOWN;
     const char *pgpbin = rpmGetPath("%{_pgpbin}", NULL);
 
-    if (!pgp_version)
-    {
+    if (saved_pgp_version == PGP_UNKNOWN) {
 	char *pgpvbin;
 	struct stat statbuf;
 	
-	if (!pgpbin || ! (pgpvbin = (char *)malloc(strlen(pgpbin) + 2)))
-	{
-	  pgp_version = -1;
+	if (!pgpbin || ! (pgpvbin = (char *)alloca(strlen(pgpbin) + 2))) {
+	  saved_pgp_version = -1;
 	  return NULL;
 	}
 	sprintf(pgpvbin, "%sv", pgpbin);
 
-	/* XXX Prefer pgp-2.6.3 over pgp5 */
-	if (stat(pgpbin, &statbuf) == 0)
-	  pgp_version = 26;
-	else if (stat(pgpvbin, &statbuf) == 0)
-	  pgp_version = 50;
+	if (stat(pgpvbin, &statbuf) == 0)
+	  saved_pgp_version = PGP_5;
+	else if (stat(pgpbin, &statbuf) == 0)
+	  saved_pgp_version = PGP_2;
 	else
-	  pgp_version = -1;
-
-    	free(pgpvbin);
+	  saved_pgp_version = PGP_NOTDETECTED;
     }
 
-    switch (sigTag)
-    {
-	case RPMSIGTAG_PGP:
-	    if (pgp_version == 26)
-	        return pgpbin;
-	    break;
-	case RPMSIGTAG_PGP5:
-	    if (pgp_version == 50)
-		return pgpbin;
-	    break;
-	default:
-	    break;
+    if (pgpbin && pgpVer)
+	*pgpVer = saved_pgp_version;
+    return pgpbin;
+}
+
+static int checkSize(FD_t fd, int size, int sigsize)
+{
+    int headerArchiveSize;
+    struct stat statbuf;
+
+    fstat(fdFileno(fd), &statbuf);
+
+    if (S_ISREG(statbuf.st_mode)) {
+	headerArchiveSize = statbuf.st_size - sizeof(struct rpmlead) - sigsize;
+
+	rpmMessage(RPMMESS_DEBUG, _("sigsize         : %d\n"), sigsize);
+	rpmMessage(RPMMESS_DEBUG, _("Header + Archive: %d\n"), headerArchiveSize);
+	rpmMessage(RPMMESS_DEBUG, _("expected size   : %d\n"), size);
+
+	return size - headerArchiveSize;
+    } else {
+	rpmMessage(RPMMESS_DEBUG, _("file is not regular -- skipping size check\n"));
+	return 0;
     }
-    return NULL;
 }
 
 /* rpmReadSignature() emulates the new style signatures if it finds an */
 /* old-style one.  It also immediately verifies the header+archive  */
 /* size and returns an error if it doesn't match.                   */
 
-int rpmReadSignature(FD_t fd, Header *header, short sig_type)
+int rpmReadSignature(FD_t fd, Header *headerp, short sig_type)
 {
     unsigned char buf[2048];
     int sigSize, pad;
@@ -138,9 +126,8 @@ int rpmReadSignature(FD_t fd, Header *header, short sig_type)
     int_32 *archSize;
     Header h;
 
-    if (header) {
-	*header = NULL;
-    }
+    if (headerp)
+	*headerp = NULL;
     
     switch (sig_type) {
       case RPMSIG_NONE:
@@ -149,12 +136,11 @@ int rpmReadSignature(FD_t fd, Header *header, short sig_type)
       case RPMSIG_PGP262_1024:
 	rpmMessage(RPMMESS_DEBUG, _("Old PGP signature\n"));
 	/* These are always 256 bytes */
-	if (timedRead(fd, buf, 256) != 256) {
+	if (timedRead(fd, buf, 256) != 256)
 	    return 1;
-	}
-	if (header) {
-	    *header = headerNew();
-	    headerAddEntry(*header, RPMSIGTAG_PGP, RPM_BIN_TYPE, buf, 152);
+	if (headerp) {
+	    *headerp = headerNew();
+	    headerAddEntry(*headerp, RPMSIGTAG_PGP, RPM_BIN_TYPE, buf, 152);
 	}
 	break;
       case RPMSIG_MD5:
@@ -162,14 +148,13 @@ int rpmReadSignature(FD_t fd, Header *header, short sig_type)
 	rpmError(RPMERR_BADSIGTYPE,
 	      _("Old (internal-only) signature!  How did you get that!?"));
 	return 1;
-	break;
+	/*@notreached@*/ break;
       case RPMSIG_HEADERSIG:
 	rpmMessage(RPMMESS_DEBUG, _("New Header signature\n"));
 	/* This is a new style signature */
 	h = headerRead(fd, HEADER_MAGIC_YES);
-	if (h == NULL) {
+	if (h == NULL)
 	    return 1;
-	}
 	sigSize = headerSizeof(h, HEADER_MAGIC_YES);
 	pad = (8 - (sigSize % 8)) % 8; /* 8-byte pad */
 	rpmMessage(RPMMESS_DEBUG, _("Signature size: %d\n"), sigSize);
@@ -188,8 +173,8 @@ int rpmReadSignature(FD_t fd, Header *header, short sig_type)
 		return 1;
 	    }
 	}
-	if (header) {
-	    *header = h;
+	if (headerp) {
+	    *headerp = h;
 	} else {
 	    headerFree(h);
 	}
@@ -205,22 +190,28 @@ int rpmWriteSignature(FD_t fd, Header header)
 {
     int sigSize, pad;
     unsigned char buf[8];
+    int rc = 0;
     
-    headerWrite(fd, header, HEADER_MAGIC_YES);
+    rc = headerWrite(fd, header, HEADER_MAGIC_YES);
+    if (rc)
+	return rc;
+
     sigSize = headerSizeof(header, HEADER_MAGIC_YES);
     pad = (8 - (sigSize % 8)) % 8;
     if (pad) {
 	rpmMessage(RPMMESS_DEBUG, _("Signature size: %d\n"), sigSize);
 	rpmMessage(RPMMESS_DEBUG, _("Signature pad : %d\n"), pad);
 	memset(buf, 0, pad);
-	(void)fdWrite(fd, buf, pad);
+	if (fdWrite(fd, buf, pad) != pad)
+	    rc = 1;
     }
-    return 0;
+    return rc;
 }
 
 Header rpmNewSignature(void)
 {
-    return headerNew();
+    Header h = headerNew();
+    return h;
 }
 
 void rpmFreeSignature(Header h)
@@ -228,44 +219,8 @@ void rpmFreeSignature(Header h)
     headerFree(h);
 }
 
-int rpmAddSignature(Header header, const char *file, int_32 sigTag, const char *passPhrase)
-{
-    struct stat statbuf;
-    int_32 size;
-    unsigned char buf[16];
-    void *sig;
-    int ret = -1;
-    
-    switch (sigTag) {
-      case RPMSIGTAG_SIZE:
-	stat(file, &statbuf);
-	size = statbuf.st_size;
-	ret = 0;
-	headerAddEntry(header, RPMSIGTAG_SIZE, RPM_INT32_TYPE, &size, 1);
-	break;
-      case RPMSIGTAG_MD5:
-	ret = mdbinfile(file, buf);
-	if (ret == 0)
-	    headerAddEntry(header, sigTag, RPM_BIN_TYPE, buf, 16);
-	break;
-      case RPMSIGTAG_PGP:
-      case RPMSIGTAG_PGP5:
-	ret = makePGPSignature(file, &sig, &size, passPhrase, sigTag);
-	if (ret == 0)
-	    headerAddEntry(header, sigTag, RPM_BIN_TYPE, sig, size);
-	break;
-      case RPMSIGTAG_GPG:
-        ret = makeGPGSignature(file, &sig, &size, passPhrase);
-	if (ret == 0)
-	    headerAddEntry(header, sigTag, RPM_BIN_TYPE, sig, size);
-	break;
-    }
-
-    return ret;
-}
-
-static int makePGPSignature(const char *file, void **sig, int_32 *size,
-			    const char *passPhrase, int sigTag)
+static int makePGPSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_32 *size,
+			    const char *passPhrase)
 {
     char sigfile[1024];
     int pid, status;
@@ -274,12 +229,14 @@ static int makePGPSignature(const char *file, void **sig, int_32 *size,
 
     sprintf(sigfile, "%s.sig", file);
 
+    inpipe[0] = inpipe[1] = 0;
     pipe(inpipe);
     
     if (!(pid = fork())) {
 	const char *pgp_path = rpmExpand("%{_pgp_path}", NULL);
 	const char *name = rpmExpand("+myname=\"%{_pgp_name}\"", NULL);
 	const char *path;
+	pgpVersion pgpVer;
 
 	close(STDIN_FILENO);
 	dup2(inpipe[0], 3);
@@ -291,19 +248,22 @@ static int makePGPSignature(const char *file, void **sig, int_32 *size,
 
 	/* dosetenv("PGPPASS", passPhrase, 1); */
 
-	if ((path = rpmDetectPGPVersion(sigTag)) != NULL) {
-	    switch(sigTag) {
-	    case RPMSIGTAG_PGP:
+	if ((path = rpmDetectPGPVersion(&pgpVer)) != NULL) {
+	    switch(pgpVer) {
+	    case PGP_2:
 		execlp(path, "pgp", "+batchmode=on", "+verbose=0", "+armor=off",
 		    name, "-sb", file, sigfile, NULL);
 		break;
-	    case RPMSIGTAG_PGP5:
+	    case PGP_5:
 		execlp(path,"pgps", "+batchmode=on", "+verbose=0", "+armor=off",
 		    name, "-b", file, "-o", sigfile, NULL);
 		break;
+	    case PGP_UNKNOWN:
+	    case PGP_NOTDETECTED:
+		break;
 	    }
 	}
-	rpmError(RPMERR_EXEC, _("Couldn't exec pgp"));
+	rpmError(RPMERR_EXEC, _("Couldn't exec pgp (%s)"), path);
 	_exit(RPMERR_EXEC);
     }
 
@@ -327,7 +287,7 @@ static int makePGPSignature(const char *file, void **sig, int_32 *size,
 
     *size = statbuf.st_size;
     rpmMessage(RPMMESS_DEBUG, _("PGP sig size: %d\n"), *size);
-    *sig = malloc(*size);
+    *sig = xmalloc(*size);
     
     {	FD_t fd;
 	int rc;
@@ -352,7 +312,7 @@ static int makePGPSignature(const char *file, void **sig, int_32 *size,
  * but this could be a good place to start looking if errors in GPG signature
  * creation crop up.
  */
-static int makeGPGSignature(const char *file, void **sig, int_32 *size,
+static int makeGPGSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_32 *size,
 			    const char *passPhrase)
 {
     char sigfile[1024];
@@ -363,6 +323,7 @@ static int makeGPGSignature(const char *file, void **sig, int_32 *size,
 
     sprintf(sigfile, "%s.sig", file);
 
+    inpipe[0] = inpipe[1] = 0;
     pipe(inpipe);
     
     if (!(pid = fork())) {
@@ -403,7 +364,7 @@ static int makeGPGSignature(const char *file, void **sig, int_32 *size,
 
     *size = statbuf.st_size;
     rpmMessage(RPMMESS_DEBUG, _("GPG sig size: %d\n"), *size);
-    *sig = malloc(*size);
+    *sig = xmalloc(*size);
     
     {	FD_t fd;
 	int rc;
@@ -423,60 +384,42 @@ static int makeGPGSignature(const char *file, void **sig, int_32 *size,
     return 0;
 }
 
-static int checkSize(FD_t fd, int size, int sigsize)
+int rpmAddSignature(Header header, const char *file, int_32 sigTag, const char *passPhrase)
 {
-    int headerArchiveSize;
     struct stat statbuf;
-
-    fstat(fdFileno(fd), &statbuf);
-
-    if (S_ISREG(statbuf.st_mode)) {
-	headerArchiveSize = statbuf.st_size - sizeof(struct rpmlead) - sigsize;
-
-	rpmMessage(RPMMESS_DEBUG, _("sigsize         : %d\n"), sigsize);
-	rpmMessage(RPMMESS_DEBUG, _("Header + Archive: %d\n"), headerArchiveSize);
-	rpmMessage(RPMMESS_DEBUG, _("expected size   : %d\n"), size);
-
-	return size - headerArchiveSize;
-    } else {
-	rpmMessage(RPMMESS_DEBUG, _("file is not regular -- skipping size check\n"));
-	return 0;
-    }
-}
-
-int rpmVerifySignature(const char *file, int_32 sigTag, void *sig, int count,
-		    char *result)
-{
+    int_32 size;
+    unsigned char buf[16];
+    void *sig;
+    int ret = -1;
+    
     switch (sigTag) {
-      case RPMSIGTAG_SIZE:
-	if (verifySizeSignature(file, *(int_32 *)sig, result)) {
-	    return RPMSIG_BAD;
-	}
+    case RPMSIGTAG_SIZE:
+	stat(file, &statbuf);
+	size = statbuf.st_size;
+	ret = 0;
+	headerAddEntry(header, RPMSIGTAG_SIZE, RPM_INT32_TYPE, &size, 1);
 	break;
-      case RPMSIGTAG_MD5:
-	if (verifyMD5Signature(file, sig, result, mdbinfile)) {
-	    return 1;
-	}
+    case RPMSIGTAG_MD5:
+	ret = mdbinfile(file, buf);
+	if (ret == 0)
+	    headerAddEntry(header, sigTag, RPM_BIN_TYPE, buf, 16);
 	break;
-      case RPMSIGTAG_LEMD5_1:
-      case RPMSIGTAG_LEMD5_2:
-	if (verifyMD5Signature(file, sig, result, mdbinfileBroken)) {
-	    return 1;
-	}
+    case RPMSIGTAG_PGP5:	/* XXX legacy */
+    case RPMSIGTAG_PGP:
+	rpmMessage(RPMMESS_VERBOSE, _("Generating signature using PGP.\n"));
+	ret = makePGPSignature(file, &sig, &size, passPhrase);
+	if (ret == 0)
+	    headerAddEntry(header, sigTag, RPM_BIN_TYPE, sig, size);
 	break;
-      case RPMSIGTAG_PGP:
-      case RPMSIGTAG_PGP5:
-	return verifyPGPSignature(file, sig, count, result, sigTag);
+    case RPMSIGTAG_GPG:
+	rpmMessage(RPMMESS_VERBOSE, _("Generating signature using GPG.\n"));
+        ret = makeGPGSignature(file, &sig, &size, passPhrase);
+	if (ret == 0)
+	    headerAddEntry(header, sigTag, RPM_BIN_TYPE, sig, size);
 	break;
-      case RPMSIGTAG_GPG:
-	return verifyGPGSignature(file, sig, count, result);
-	break;
-      default:
-	sprintf(result, "Do not know how to verify sig type %d\n", sigTag);
-	return RPMSIG_UNKNOWN;
     }
 
-    return RPMSIG_OK;
+    return ret;
 }
 
 static int verifySizeSignature(const char *datafile, int_32 size, char *result)
@@ -495,6 +438,8 @@ static int verifySizeSignature(const char *datafile, int_32 size, char *result)
     return 0;
 }
 
+#define	X(_x)	(unsigned)((_x) & 0xff)
+
 static int verifyMD5Signature(const char *datafile, unsigned char *sig, 
 			      char *result, md5func fn)
 {
@@ -507,29 +452,29 @@ static int verifyMD5Signature(const char *datafile, unsigned char *sig,
 		"%02x%02x%02x%02x%02x\n"
 		"Saw     : %02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x"
 		"%02x%02x%02x%02x%02x\n",
-		sig[0],  sig[1],  sig[2],  sig[3],
-		sig[4],  sig[5],  sig[6],  sig[7],
-		sig[8],  sig[9],  sig[10], sig[11],
-		sig[12], sig[13], sig[14], sig[15],
-		md5sum[0],  md5sum[1],  md5sum[2],  md5sum[3],
-		md5sum[4],  md5sum[5],  md5sum[6],  md5sum[7],
-		md5sum[8],  md5sum[9],  md5sum[10], md5sum[11],
-		md5sum[12], md5sum[13], md5sum[14], md5sum[15]);
+		X(sig[0]),  X(sig[1]),  X(sig[2]),  X(sig[3]),
+		X(sig[4]),  X(sig[5]),  X(sig[6]),  X(sig[7]),
+		X(sig[8]),  X(sig[9]),  X(sig[10]), X(sig[11]),
+		X(sig[12]), X(sig[13]), X(sig[14]), X(sig[15]),
+		X(md5sum[0]),  X(md5sum[1]),  X(md5sum[2]),  X(md5sum[3]),
+		X(md5sum[4]),  X(md5sum[5]),  X(md5sum[6]),  X(md5sum[7]),
+		X(md5sum[8]),  X(md5sum[9]),  X(md5sum[10]), X(md5sum[11]),
+		X(md5sum[12]), X(md5sum[13]), X(md5sum[14]), X(md5sum[15]) );
 	return 1;
     }
 
     sprintf(result, "MD5 sum OK: %02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x"
                     "%02x%02x%02x%02x%02x\n",
-	    md5sum[0],  md5sum[1],  md5sum[2],  md5sum[3],
-	    md5sum[4],  md5sum[5],  md5sum[6],  md5sum[7],
-	    md5sum[8],  md5sum[9],  md5sum[10], md5sum[11],
-	    md5sum[12], md5sum[13], md5sum[14], md5sum[15]);
+	    X(md5sum[0]),  X(md5sum[1]),  X(md5sum[2]),  X(md5sum[3]),
+	    X(md5sum[4]),  X(md5sum[5]),  X(md5sum[6]),  X(md5sum[7]),
+	    X(md5sum[8]),  X(md5sum[9]),  X(md5sum[10]), X(md5sum[11]),
+	    X(md5sum[12]), X(md5sum[13]), X(md5sum[14]), X(md5sum[15]) );
 
     return 0;
 }
 
 static int verifyPGPSignature(const char *datafile, void *sig,
-			      int count, char *result, int sigTag)
+			      int count, char *result)
 {
     int pid, status, outpipe[2];
     FD_t sfd;
@@ -537,29 +482,24 @@ static int verifyPGPSignature(const char *datafile, void *sig,
     unsigned char buf[8192];
     FILE *file;
     int res = RPMSIG_OK;
-    int usingPGP5 = 0;
     const char *path;
+    pgpVersion pgpVer;
 
     /* What version do we have? */
-    if ((path = rpmDetectPGPVersion(RPMSIGTAG_PGP5))/* Use pgp5 if we have it */
-	    || sigTag == RPMSIGTAG_PGP5)	    /* ... or request it.     */
-    {
-	usingPGP5 = 1;
-	/* Its sad but true: pgp-5.0 returns also an exit value of 0  */
-	/* when it finds a BAD signature.  So instead we have to use  */
-	/* the text output.                                           */
-	res = RPMSIG_BAD;
-    }
-    else if (! (path = rpmDetectPGPVersion(RPMSIGTAG_PGP))
-	    || sigTag != RPMSIGTAG_PGP)
-	path = NULL;	/* Fail */
-    if (path == NULL)
-    {
+    if ((path = rpmDetectPGPVersion(&pgpVer)) == NULL) {
 	errno = ENOENT;
 	rpmError(RPMERR_EXEC, 
 		 _("Could not run pgp.  Use --nopgp to skip PGP checks."));
 	_exit(RPMERR_EXEC);
     }
+
+    /*
+     * Sad but true: pgp-5.0 returns exit value of 0 on bad signature.
+     * Instead we have to use the text output to detect a bad signature.
+     */
+    if (pgpVer == PGP_5)
+	res = RPMSIG_BAD;
+
     /* Write out the signature */
   { const char *tmppath = rpmGetPath("%{_tmppath}", NULL);
     sigfile = tempnam(tmppath, "rpmsig");
@@ -570,6 +510,7 @@ static int verifyPGPSignature(const char *datafile, void *sig,
     fdClose(sfd);
 
     /* Now run PGP */
+    outpipe[0] = outpipe[1] = 0;
     pipe(outpipe);
 
     if (!(pid = fork())) {
@@ -582,11 +523,12 @@ static int verifyPGPSignature(const char *datafile, void *sig,
 	if (pgp_path && *pgp_path != '%')
 	    dosetenv("PGPPATH", pgp_path, 1);
 
-	if (usingPGP5) {
+	switch (pgpVer) {
+	case PGP_5:
 	    /* Some output (in particular "This signature applies to */
 	    /* another message") is _always_ written to stderr; we   */
 	    /* want to catch that output, so dup stdout to stderr:   */
-	    int save_stderr = dup(2);
+	{   int save_stderr = dup(2);
 	    dup2(1, 2);
 	    execlp(path, "pgpv", "+batchmode=on", "+verbose=0",
 		   /* Write "Good signature..." to stdout: */
@@ -597,9 +539,14 @@ static int verifyPGPSignature(const char *datafile, void *sig,
 	    /* Restore stderr so we can print the error message below. */
 	    dup2(save_stderr, 2);
 	    close(save_stderr);
-	} else {
+	}   break;
+	case PGP_2:
 	    execlp(path, "pgp", "+batchmode=on", "+verbose=0",
 		   sigfile, datafile, NULL);
+	    break;
+	case PGP_UNKNOWN:
+	case PGP_NOTDETECTED:
+	    break;
 	}
 
 	fprintf(stderr, _("exec failed!\n"));
@@ -658,6 +605,7 @@ static int verifyGPGSignature(const char *datafile, void *sig,
     fdClose(sfd);
 
     /* Now run GPG */
+    outpipe[0] = outpipe[1] = 0;
     pipe(outpipe);
 
     if (!(pid = fork())) {
@@ -700,62 +648,13 @@ static int verifyGPGSignature(const char *datafile, void *sig,
     return res;
 }
 
-char *rpmGetPassPhrase(const char *prompt, const int sigTag)
-{
-    char *pass;
-    int aok;
-
-    switch (sigTag) {
-      case RPMSIGTAG_GPG:
-      { const char *name = rpmExpand("%{_gpg_name}", NULL);
-	aok = (name && *name != '%');
-	xfree(name);
-      }
-	if (!aok) {
-	    rpmError(RPMERR_SIGGEN,
-		_("You must set \"%%_gpg_name\" in your macro file"));
-	    return NULL;
-	}
-	break;
-      case RPMSIGTAG_PGP: 
-      case RPMSIGTAG_PGP5: 
-      { const char *name = rpmExpand("%{_pgp_name}", NULL);
-	aok = (name && *name != '%');
-	xfree(name);
-      }
-	if (!aok) {
-	    rpmError(RPMERR_SIGGEN,
-		_("You must set \"%%_pgp_name\" in your macro file"));
-	    return NULL;
-	}
-	break;
-      default:
-	/* Currently the calling function (rpm.c:main) is checking this and
-	 * doing a better job.  This section should never be accessed.
-	 */
-	rpmError(RPMERR_SIGGEN, _("Invalid %%_signature spec in macro file"));
-	return NULL;
-    }
-
-    if (prompt) {
-	pass = getpass(prompt);
-    } else {
-	pass = getpass("");
-    }
-
-    if (checkPassPhrase(pass, sigTag)) {
-	return NULL;
-    }
-
-    return pass;
-}
-
 static int checkPassPhrase(const char *passPhrase, const int sigTag)
 {
     int passPhrasePipe[2];
     int pid, status;
     int fd;
 
+    passPhrasePipe[0] = passPhrasePipe[1] = 0;
     pipe(passPhrasePipe);
     if (!(pid = fork())) {
 	close(STDIN_FILENO);
@@ -786,36 +685,40 @@ static int checkPassPhrase(const char *passPhrase, const int sigTag)
 	           NULL);
 	    rpmError(RPMERR_EXEC, _("Couldn't exec gpg"));
 	    _exit(RPMERR_EXEC);
-	}   break;
+	}   /*@notreached@*/ break;
+	case RPMSIGTAG_PGP5:	/* XXX legacy */
 	case RPMSIGTAG_PGP:
-	case RPMSIGTAG_PGP5:
 	{   const char *pgp_path = rpmExpand("%{_pgp_path}", NULL);
 	    const char *name = rpmExpand("+myname=\"%{_pgp_name}\"", NULL);
 	    const char *path;
+	    pgpVersion pgpVer;
 
 	    dosetenv("PGPPASSFD", "3", 1);
 	    if (pgp_path && *pgp_path != '%')
 		dosetenv("PGPPATH", pgp_path, 1);
 
-	    if ((path = rpmDetectPGPVersion(sigTag)) != NULL) {
-		switch(sigTag) {
-		case RPMSIGTAG_PGP:
+	    if ((path = rpmDetectPGPVersion(&pgpVer)) != NULL) {
+		switch(pgpVer) {
+		case PGP_2:
 		    execlp(path, "pgp", "+batchmode=on", "+verbose=0",
 			name, "-sf", NULL);
 		    break;
-		case RPMSIGTAG_PGP5:
+		case PGP_5:	/* XXX legacy */
 		    execlp(path,"pgps", "+batchmode=on", "+verbose=0",
 			name, "-f", NULL);
 		    break;
+		case PGP_UNKNOWN:
+		case PGP_NOTDETECTED:
+		    break;
 		}
 	    }
 	    rpmError(RPMERR_EXEC, _("Couldn't exec pgp"));
 	    _exit(RPMERR_EXEC);
-	}   break;
+	}   /*@notreached@*/ break;
 	default: /* This case should have been screened out long ago. */
 	    rpmError(RPMERR_SIGGEN, _("Invalid %%_signature spec in macro file"));
 	    _exit(RPMERR_SIGGEN);
-	    break;
+	    /*@notreached@*/ break;
 	}
     }
 
@@ -832,3 +735,84 @@ static int checkPassPhrase(const char *passPhrase, const int sigTag)
     /* passPhrase is good */
     return 0;
 }
+
+char *rpmGetPassPhrase(const char *prompt, const int sigTag)
+{
+    char *pass;
+    int aok;
+
+    switch (sigTag) {
+    case RPMSIGTAG_GPG:
+      { const char *name = rpmExpand("%{_gpg_name}", NULL);
+	aok = (name && *name != '%');
+	xfree(name);
+      }
+	if (!aok) {
+	    rpmError(RPMERR_SIGGEN,
+		_("You must set \"%%_gpg_name\" in your macro file"));
+	    return NULL;
+	}
+	break;
+    case RPMSIGTAG_PGP5: 	/* XXX legacy */
+    case RPMSIGTAG_PGP: 
+      { const char *name = rpmExpand("%{_pgp_name}", NULL);
+	aok = (name && *name != '%');
+	xfree(name);
+      }
+	if (!aok) {
+	    rpmError(RPMERR_SIGGEN,
+		_("You must set \"%%_pgp_name\" in your macro file"));
+	    return NULL;
+	}
+	break;
+    default:
+	/* Currently the calling function (rpm.c:main) is checking this and
+	 * doing a better job.  This section should never be accessed.
+	 */
+	rpmError(RPMERR_SIGGEN, _("Invalid %%_signature spec in macro file"));
+	return NULL;
+	/*@notreached@*/ break;
+    }
+
+    pass = /*@-unrecog@*/ getpass( (prompt ? prompt : "") ) /*@=unrecog@*/ ;
+
+    if (checkPassPhrase(pass, sigTag))
+	return NULL;
+
+    return pass;
+}
+
+int rpmVerifySignature(const char *file, int_32 sigTag, void *sig, int count,
+		    char *result)
+{
+    switch (sigTag) {
+    case RPMSIGTAG_SIZE:
+	if (verifySizeSignature(file, *(int_32 *)sig, result)) {
+	    return RPMSIG_BAD;
+	}
+	break;
+    case RPMSIGTAG_MD5:
+	if (verifyMD5Signature(file, sig, result, mdbinfile)) {
+	    return 1;
+	}
+	break;
+    case RPMSIGTAG_LEMD5_1:
+    case RPMSIGTAG_LEMD5_2:
+	if (verifyMD5Signature(file, sig, result, mdbinfileBroken)) {
+	    return 1;
+	}
+	break;
+    case RPMSIGTAG_PGP5:	/* XXX legacy */
+    case RPMSIGTAG_PGP:
+	return verifyPGPSignature(file, sig, count, result);
+	/*@notreached@*/ break;
+    case RPMSIGTAG_GPG:
+	return verifyGPGSignature(file, sig, count, result);
+	/*@notreached@*/ break;
+    default:
+	sprintf(result, "Do not know how to verify sig type %d\n", sigTag);
+	return RPMSIG_UNKNOWN;
+    }
+
+    return RPMSIG_OK;
+}
