@@ -1,6 +1,6 @@
 #include "system.h"
 
-#include "rpmlib.h"
+#include <rpmlib.h>
 
 #include "depends.h"
 #include "install.h"
@@ -8,29 +8,74 @@
 #include "misc.h"
 #include "rpmdb.h"
 
-static char * SCRIPT_PATH = "PATH=/sbin:/bin:/usr/sbin:/usr/bin:"
-			                 "/usr/X11R6/bin";
+static char * SCRIPT_PATH = "PATH=/sbin:/bin:/usr/sbin:/usr/bin:/usr/X11R6/bin";
 
-static int removeFile(char * file, unsigned int flags, short mode, 
-		      enum fileActions action);
-static int runScript(Header h, const char * root, int progArgc, const char ** progArgv, 
-		     const char * script, int arg1, int arg2, FD_t errfd);
+static int removeFile(const char * file, unsigned int flags, short mode, 
+		      enum fileActions action)
+{
+    int rc = 0;
+    char * newfile;
+	
+    switch (action) {
 
-int removeBinaryPackage(char * prefix, rpmdb db, unsigned int offset, 
-			int flags, enum fileActions * actions, FD_t scriptFd) {
-    Header h;
+      case FA_BACKUP:
+	newfile = alloca(strlen(file) + 20);
+	strcpy(newfile, file);
+	strcat(newfile, ".rpmsave");
+	if (rename(file, newfile)) {
+	    rpmError(RPMERR_RENAME, _("rename of %s to %s failed: %s"),
+			file, newfile, strerror(errno));
+	    rc = 1;
+	}
+	break;
+
+      case FA_REMOVE:
+	if (S_ISDIR(mode)) {
+	    if (rmdir(file)) {
+		if (errno == ENOTEMPTY)
+		    rpmError(RPMERR_RMDIR, 
+			_("cannot remove %s - directory not empty"), 
+			file);
+		else
+		    rpmError(RPMERR_RMDIR, _("rmdir of %s failed: %s"),
+				file, strerror(errno));
+		rc = 1;
+	    }
+	} else {
+	    if (unlink(file)) {
+		if (errno != ENOENT || !(flags & RPMFILE_MISSINGOK)) {
+		    rpmError(RPMERR_UNLINK, 
+			      _("removal of %s failed: %s"),
+				file, strerror(errno));
+		}
+		rc = 1;
+	    }
+	}
+	break;
+      case FA_UNKNOWN:
+      case FA_CREATE:
+      case FA_SAVE:
+      case FA_ALTNAME:
+      case FA_SKIP:
+      case FA_SKIPNSTATE:
+      case FA_SKIPNETSHARED:
+	break;
+    }
+ 
+    return 0;
+}
+
+int removeBinaryPackage(const char * prefix, rpmdb db, unsigned int offset, 
+			int flags, enum fileActions * actions, FD_t scriptFd)
+{
+    Header h = NULL;
     int i;
     int fileCount;
-    char * name, * version, * release;
-    char * fnbuffer = NULL;
-    dbiIndexSet matches;
-    int fnbuffersize = 0;
-    int prefixLength = strlen(prefix);
-    char ** fileList, ** fileMd5List;
-    int type, count;
-    uint_32 * fileFlagsList;
-    int_16 * fileModesList;
+    const char * name, * version, * release;
+    const char ** fileList;
+    int type;
     int scriptArg;
+    int rc = 0;
 
     if (flags & RPMTRANS_FLAG_JUSTDB)
 	flags |= RPMTRANS_FLAG_NOSCRIPTS;
@@ -39,51 +84,78 @@ int removeBinaryPackage(char * prefix, rpmdb db, unsigned int offset,
     if (h == NULL) {
 	rpmError(RPMERR_DBCORRUPT, _("cannot read header at %d for uninstall"),
 	      offset);
-	return 1;
+	rc = 1;
+	goto exit;
     }
 
-    headerGetEntry(h, RPMTAG_NAME, &type, (void **) &name,  &count);
-    headerGetEntry(h, RPMTAG_VERSION, &type, (void **) &version,  &count);
-    headerGetEntry(h, RPMTAG_RELEASE, &type, (void **) &release,  &count);
+    headerNVR(h, &name, &version, &release);
+
     /* when we run scripts, we pass an argument which is the number of 
        versions of this package that will be installed when we are finished */
-    if (rpmdbFindPackage(db, name, &matches)) {
-	rpmError(RPMERR_DBCORRUPT, _("cannot read packages named %s for uninstall"),
-	      name);
-	return 1;
+    {	dbiIndexSet matches;
+	if (rpmdbFindPackage(db, name, &matches)) {
+	    rpmError(RPMERR_DBCORRUPT, _("cannot read packages named %s for uninstall"),
+		name);
+	    dbiFreeIndexRecord(matches);
+	    rc = 1;
+	    goto exit;
+	}
+	scriptArg = dbiIndexSetCount(matches) - 1;
+	dbiFreeIndexRecord(matches);
     }
- 
-    scriptArg = dbiIndexSetCount(matches) - 1;
-    dbiFreeIndexRecord(matches);
 
     if (!(flags & RPMTRANS_FLAG_NOTRIGGERS)) {
 	/* run triggers from this package which are keyed on installed 
 	   packages */
 	if (runImmedTriggers(prefix, db, RPMSENSE_TRIGGERUN, h, -1, scriptFd)) {
-	    return 2;
+	    rc = 2;
+	    goto exit;
 	}
 
 	/* run triggers which are set off by the removal of this package */
-	if (runTriggers(prefix, db, RPMSENSE_TRIGGERUN, h, -1, scriptFd))
-	    return 1;
+	if (runTriggers(prefix, db, RPMSENSE_TRIGGERUN, h, -1, scriptFd)) {
+	    rc = 1;
+	    goto exit;
+	}
     }
 
     if (!(flags & RPMTRANS_FLAG_TEST)) {
 	if (runInstScript(prefix, h, RPMTAG_PREUN, RPMTAG_PREUNPROG, scriptArg, 
 		          flags & RPMTRANS_FLAG_NOSCRIPTS, scriptFd)) {
-	    headerFree(h);
-	    return 1;
+	    rc = 1;
+	    goto exit;
 	}
     }
     
     rpmMessage(RPMMESS_DEBUG, _("will remove files test = %d\n"), 
 		flags & RPMTRANS_FLAG_TEST);
+
     if (!(flags & RPMTRANS_FLAG_JUSTDB) &&
 	headerGetEntry(h, RPMTAG_FILENAMES, &type, (void **) &fileList, 
 	               &fileCount)) {
-	if (prefix[0]) {
-	    fnbuffersize = 1024;
+	const char ** fileMd5List;
+	uint_32 * fileFlagsList;
+	int_16 * fileModesList;
+	char * fnbuffer = NULL;
+	int prefixlen = 0;
+
+	/* Get alloca buffer for largest possible prefix + filename. */
+	if (prefix && prefix[0] != '\0') {
+	    int fnbuffersize = 0;
+	    size_t fnlen;
+
+	    prefixlen = strlen(prefix);
+	    for (i = 0; i < fileCount; i++) {
+		if ((fnlen = strlen(fileList[i])) > fnbuffersize)
+		    fnbuffersize = fnlen;
+	    }
+	    fnbuffersize += strlen(prefix) + sizeof("/");
 	    fnbuffer = alloca(fnbuffersize);
+	    strcpy(fnbuffer, prefix);
+	    if (fnbuffer[prefixlen-1] != '/') {
+		fnbuffer[prefixlen++] = '/';
+		fnbuffer[prefixlen] = '\0';
+	    }
 	}
 
 	headerGetEntry(h, RPMTAG_FILEMD5S, &type, (void **) &fileMd5List, 
@@ -93,27 +165,22 @@ int removeBinaryPackage(char * prefix, rpmdb db, unsigned int offset,
 	headerGetEntry(h, RPMTAG_FILEMODES, &type, (void **) &fileModesList, 
 		 &fileCount);
 
-	/* go through the filelist backwards to help insure that rmdir()
-	   will work */
+	/* Traverse filelist backwards to help insure that rmdir() will work. */
 	for (i = fileCount - 1; i >= 0; i--) {
-	    if (strcmp(prefix, "/")) {
-		if ((strlen(fileList[i]) + prefixLength + 1) > fnbuffersize) {
-		    fnbuffersize = (strlen(fileList[i]) + prefixLength) * 2;
-		    fnbuffer = alloca(fnbuffersize);
-		}
-		strcpy(fnbuffer, prefix);
-		strcat(fnbuffer, "/");
-		strcat(fnbuffer, fileList[i]);
-	    } else {
-		fnbuffer = fileList[i];
+	    const char * fn;
+
+	    fn = fileList[i];
+	    if (prefixlen) {
+		if (*fn == '/') fn++;
+		strcpy(fnbuffer + prefixlen, fn);
+		fn = fnbuffer;
 	    }
 
 	    rpmMessage(RPMMESS_DEBUG, _("   file: %s action: %s\n"),
-			fnbuffer, fileActionString(actions[i]));
+			fn, fileActionString(actions[i]));
 
 	    if (!(flags & RPMTRANS_FLAG_TEST))
-		removeFile(fnbuffer, fileFlagsList[i], fileModesList[i], 
-			   actions[i]);
+		removeFile(fn, fileFlagsList[i], fileModesList[i], actions[i]);
 	}
 
 	free(fileList);
@@ -127,37 +194,42 @@ int removeBinaryPackage(char * prefix, rpmdb db, unsigned int offset,
     }
 
     if (!(flags & RPMTRANS_FLAG_NOTRIGGERS)) {
-	/* Run postun triggers which are set off by this package's removal */
+	/* Run postun triggers which are set off by this package's removal. */
 	if (runTriggers(prefix, db, RPMSENSE_TRIGGERPOSTUN, h, -1, scriptFd)) {
-	    return 2;
+	    rc = 2;
+	    goto exit;
 	}
     }
 
-    headerFree(h);
-
     rpmMessage(RPMMESS_DEBUG, _("removing database entry\n"));
     if (!(flags & RPMTRANS_FLAG_TEST))
 	rpmdbRemove(db, offset, 0);
 
-    return 0;
+    rc = 0;
+
+ exit:
+    if (h) {
+	headerFree(h);
+    }
+    return rc;
 }
 
 static int runScript(Header h, const char * root, int progArgc, const char ** progArgv, 
-		     const char * script, int arg1, int arg2, FD_t errfd) {
-    const char ** argv;
-    int argc;
+		     const char * script, int arg1, int arg2, FD_t errfd)
+{
+    const char ** argv = NULL;
+    int argc = 0;
     const char ** prefixes = NULL;
     int numPrefixes;
     const char * oldPrefix;
     int maxPrefixLength;
     int len;
-    char * prefixBuf;
+    char * prefixBuf = NULL;
     pid_t child;
     int status;
-    const char * fn;
+    const char * fn = NULL;
     int i;
     int freePrefixes = 0;
-    int pipes[2];
     FD_t out;
 
     if (!progArgv && !script)
@@ -233,6 +305,9 @@ static int runScript(Header h, const char * root, int progArgc, const char ** pr
     }
     
     if (!(child = fork())) {
+	int pipes[2];
+
+	pipes[0] = pipes[1] = 0;
 	/* make stdin inaccessible */
 	pipe(pipes);
 	close(pipes[1]);
@@ -265,13 +340,14 @@ static int runScript(Header h, const char * root, int progArgc, const char ** pr
 	}
 
 	if (strcmp(root, "/")) {
-	    chroot(root);
+	    /*@-unrecog@*/ chroot(root); /*@=unrecog@*/
 	}
 
 	chdir("/");
 
 	execv(argv[0], (char *const *)argv);
  	_exit(-1);
+	/*@notreached@*/
     }
 
     (void)waitpid(child, &status, 0);
@@ -294,7 +370,8 @@ static int runScript(Header h, const char * root, int progArgc, const char ** pr
 }
 
 int runInstScript(const char * root, Header h, int scriptTag, int progTag,
-	          int arg, int norunScripts, FD_t err) {
+	          int arg, int norunScripts, FD_t err)
+{
     void ** programArgv;
     int programArgc;
     const char ** argv;
@@ -318,68 +395,16 @@ int runInstScript(const char * root, Header h, int scriptTag, int progTag,
     }
 
     rc = runScript(h, root, programArgc, argv, script, arg, -1, err);
-    if (programType == RPM_STRING_ARRAY_TYPE) free(programArgv);
+    if (programArgv && programType == RPM_STRING_ARRAY_TYPE) free(programArgv);
     return rc;
 }
 
-static int removeFile(char * file, unsigned int flags, short mode, 
-		      enum fileActions action) {
-    int rc = 0;
-    char * newfile;
-	
-    switch (action) {
-
-      case FA_BACKUP:
-	newfile = alloca(strlen(file) + 20);
-	strcpy(newfile, file);
-	strcat(newfile, ".rpmsave");
-	if (rename(file, newfile)) {
-	    rpmError(RPMERR_RENAME, _("rename of %s to %s failed: %s"),
-			file, newfile, strerror(errno));
-	    rc = 1;
-	}
-	break;
-
-      case FA_REMOVE:
-	if (S_ISDIR(mode)) {
-	    if (rmdir(file)) {
-		if (errno == ENOTEMPTY)
-		    rpmError(RPMERR_RMDIR, 
-			_("cannot remove %s - directory not empty"), 
-			file);
-		else
-		    rpmError(RPMERR_RMDIR, _("rmdir of %s failed: %s"),
-				file, strerror(errno));
-		rc = 1;
-	    }
-	} else {
-	    if (unlink(file)) {
-		if (errno != ENOENT || !(flags & RPMFILE_MISSINGOK)) {
-		    rpmError(RPMERR_UNLINK, 
-			      _("removal of %s failed: %s"),
-				file, strerror(errno));
-		}
-		rc = 1;
-	    }
-	}
-	break;
-      case FA_UNKNOWN:
-      case FA_CREATE:
-      case FA_SAVE:
-      case FA_SKIP:
-      case FA_SKIPNSTATE:
-      case FA_ALTNAME:
-	break;
-    }
- 
-    return 0;
-}
-
 static int handleOneTrigger(const char * root, rpmdb db, int sense, Header sourceH,
 			    Header triggeredH, int arg1correction, int arg2,
-			    char * triggersAlreadyRun, FD_t scriptFd) {
+			    char * triggersAlreadyRun, FD_t scriptFd)
+{
     const char ** triggerNames;
-    const char ** triggerVersions;
+    const char ** triggerEVR;
     const char ** triggerScripts;
     const char ** triggerProgs;
     int_32 * triggerFlags, * triggerIndices;
@@ -402,7 +427,7 @@ static int handleOneTrigger(const char * root, rpmdb db, int sense, Header sourc
     headerGetEntry(triggeredH, RPMTAG_TRIGGERFLAGS, NULL, 
 		   (void **) &triggerFlags, NULL);
     headerGetEntry(triggeredH, RPMTAG_TRIGGERVERSION, NULL, 
-		   (void **) &triggerVersions, NULL);
+		   (void **) &triggerEVR, NULL);
 
     for (i = 0; i < numTriggers; i++) {
 	if (!(triggerFlags[i] & sense)) continue;
@@ -413,14 +438,15 @@ static int handleOneTrigger(const char * root, rpmdb db, int sense, Header sourc
 	   over that here. I suspect that we'll change our minds on this
 	   and remove that, so I'm going to just 'do the right thing'. */
 	skip = strlen(triggerNames[i]);
-	if (!strncmp(triggerVersions[i], triggerNames[i], skip) &&
-	    (triggerVersions[i][skip] == '-'))
+	if (!strncmp(triggerEVR[i], triggerNames[i], skip) &&
+	    (triggerEVR[i][skip] == '-'))
 	    skip++;
 	else
 	    skip = 0;
 
-	if (!headerMatchesDepFlags(sourceH, triggerVersions[i] + skip, 
-				   triggerFlags[i])) continue;
+	if (!headerMatchesDepFlags(sourceH, triggerNames[i],
+		triggerEVR[i] + skip, triggerFlags[i]))
+	    continue;
 
 	headerGetEntry(triggeredH, RPMTAG_TRIGGERINDEX, NULL,
 		       (void **) &triggerIndices, NULL);
@@ -457,7 +483,8 @@ static int handleOneTrigger(const char * root, rpmdb db, int sense, Header sourc
 }
 
 int runTriggers(const char * root, rpmdb db, int sense, Header h,
-		int countCorrection, FD_t scriptFd) {
+		int countCorrection, FD_t scriptFd)
+{
     char * packageName;
     dbiIndexSet matches, otherMatches;
     Header triggeredH;
@@ -495,7 +522,8 @@ int runTriggers(const char * root, rpmdb db, int sense, Header h,
 }
 
 int runImmedTriggers(const char * root, rpmdb db, int sense, Header h,
-		     int countCorrection, FD_t scriptFd) {
+		     int countCorrection, FD_t scriptFd)
+{
     int rc = 0;
     dbiIndexSet matches;
     char ** triggerNames;
