@@ -1,13 +1,13 @@
 #include "system.h"
 
-#include "rpmlib.h"
+#include <rpmlib.h>
 
 #include "md5.h"
 #include "misc.h"
 #include "install.h"
 
 #include "build/rpmbuild.h"
-#include "rpmurl.h"
+#include <rpmurl.h>
 
 static int _ie = 0x44332211;
 static union _endian { int i; char b[4]; } *_endian = (union _endian *)&_ie;
@@ -19,10 +19,12 @@ static union _endian { int i; char b[4]; } *_endian = (union _endian *)&_ie;
 #define	POPT_NOFILES	1000
 
 /* ========== Verify specific popt args */
-static void verifyArgCallback(poptContext con, enum poptCallbackReason reason,
-			     const struct poptOption * opt, const char * arg, 
-			     QVA_t *qva)
+static void verifyArgCallback(/*@unused@*/poptContext con,
+	/*@unused@*/enum poptCallbackReason reason,
+	const struct poptOption * opt, /*@unused@*/const char * arg,
+	const void * data)
 {
+    QVA_t *qva = (QVA_t *) data;
     switch (opt->val) {
       case POPT_NOFILES: qva->qva_flags |= VERIFY_FILES; break;
     }
@@ -301,12 +303,12 @@ static int verifyHeader(QVA_t *qva, Header h)
     return ec;
 }
 
-static int verifyDependencies(rpmdb db, Header h) {
+static int verifyDependencies(/*@only@*/rpmdb db, Header h) {
     rpmTransactionSet rpmdep;
     struct rpmDependencyConflict * conflicts;
     int numConflicts;
     const char * name, * version, * release;
-    int type, count, i;
+    int i;
 
     rpmdep = rpmtransCreateSet(db, NULL);
     rpmtransAddPackage(rpmdep, h, NULL, NULL, 0, NULL);
@@ -315,9 +317,7 @@ static int verifyDependencies(rpmdb db, Header h) {
     rpmtransFree(rpmdep);
 
     if (numConflicts) {
-	headerGetEntry(h, RPMTAG_NAME, &type, (void **) &name, &count);
-	headerGetEntry(h, RPMTAG_VERSION, &type, (void **) &version, &count);
-	headerGetEntry(h, RPMTAG_RELEASE, &type, (void **) &release, &count);
+	headerNVR(h, &name, &version, &release);
 	fprintf(stdout, _("Unsatisfied dependencies for %s-%s-%s: "),
 		name, version, release);
 	for (i = 0; i < numConflicts; i++) {
@@ -363,7 +363,7 @@ int rpmVerify(QVA_t *qva, enum rpmQVSources source, const char *arg)
     case RPMQV_RPM:
 	if (!(qva->qva_flags & VERIFY_DEPS))
 	    break;
-	/* fall thru */
+	/*@fallthrough@*/
     default:
 	if (rpmdbOpen(qva->qva_prefix, &db, O_RDONLY, 0644)) {
 	    fprintf(stderr, _("rpmVerify: rpmdbOpen() failed\n"));
