@@ -1,15 +1,3 @@
-#ifdef __sgi
-#define SYSDIR
-#endif
-
-#ifdef __hpux
-#define SYSDIR
-#endif
-
-#if defined(sun) && !defined(__svr4__)
-#define SYSDIR
-#endif
-
 /* Copyright (C) 1991, 1992, 1993, 1994 Free Software Foundation, Inc.
 
 This library is free software; you can redistribute it and/or
@@ -32,10 +20,7 @@ Cambridge, MA 02139, USA.  */
  #pragma alloca
 #endif
 
-#include <errno.h>
-#include <sys/types.h>
-
-#include "config.h"
+#include "system.h"
 
 /* Comment out all this code if we are using the GNU C Library, and are not
    actually compiling the library itself.  This code is part of the GNU C
@@ -47,7 +32,6 @@ Cambridge, MA 02139, USA.  */
 
 #if defined (_LIBC) || !defined (__GNU_LIBRARY__)
 
-
 #ifdef	STDC_HEADERS
 #include <stddef.h>
 #endif
@@ -69,27 +53,6 @@ extern int errno;
 #define	NULL	0
 #endif
 
-#if	defined (POSIX) || defined (DIRENT) || defined (__GNU_LIBRARY__)  || defined(HAVE_DIRENT_H)
-#include <dirent.h>
-#ifndef	__GNU_LIBRARY__
-#define D_NAMLEN(d) strlen((d)->d_name)
-#else	/* GNU C library.  */
-#define D_NAMLEN(d) ((d)->d_namlen)
-#endif	/* Not GNU C library.  */
-#else	/* Not POSIX or DIRENT.  */
-#define direct dirent
-#define D_NAMLEN(d) ((d)->d_namlen)
-#ifdef	SYSNDIR
-#include <sys/ndir.h>
-#endif	/* SYSNDIR */
-#ifdef	SYSDIR
-#include <sys/dir.h>
-#endif	/* SYSDIR */
-#ifdef NDIR
-#include <ndir.h>
-#endif	/* NDIR */
-#endif	/* POSIX or DIRENT or __GNU_LIBRARY__.  */
-
 #if defined (POSIX) && !defined (__GNU_LIBRARY__)
 /* Posix does not require that the d_ino field be present, and some
    systems do not provide it. */
@@ -105,18 +68,12 @@ extern int errno;
 #else	/* No standard headers.  */
 
 #ifdef HAVE_STRING_H
-#include <string.h>
 #define	ANSI_STRING
-#else
-#include <strings.h>
 #endif
 #ifdef	HAVE_MEMORY_H
 #include <memory.h>
 #endif
 
-extern char *malloc (), *realloc ();
-extern void free ();
-
 extern void qsort ();
 extern void abort (), exit ();
 
@@ -188,22 +145,21 @@ extern char *alloca ();
 
 /* Some system header files erroneously define these.
    We want our own definitions from <fnmatch.h> to take precedence.  */
-#undef	FNM_PATHNAME
-#undef	FNM_NOESCAPE
-#undef	FNM_PERIOD
+#undef FNM_PATHNAME
+#undef FNM_NOESCAPE
+#undef FNM_PERIOD
 
 /* Some system header files erroneously define these.
    We want our own definitions from <glob.h> to take precedence.  */
-#undef	GLOB_ERR
-#undef	GLOB_MARK
-#undef	GLOB_NOSORT
-#undef	GLOB_DOOFFS
-#undef	GLOB_NOCHECK
-#undef	GLOB_APPEND
-#undef	GLOB_NOESCAPE
-#undef	GLOB_PERIOD
-#include "miscfn.h"
-
+#undef GLOB_ERR
+#undef GLOB_MARK
+#undef GLOB_NOSORT
+#undef GLOB_DOOFFS
+#undef GLOB_NOCHECK
+#undef GLOB_APPEND
+#undef GLOB_NOESCAPE
+#undef GLOB_PERIOD
+
 __ptr_t (*__glob_opendir_hook) __P ((const char *directory));
 const char *(*__glob_readdir_hook) __P ((__ptr_t stream));
 void (*__glob_closedir_hook) __P ((__ptr_t stream));
@@ -582,11 +538,7 @@ glob_in_dir (pattern, directory, flags, errfunc, pglob)
 		if (! REAL_DIR_ENTRY (d))
 		  continue;
 		name = d->d_name;
-#ifdef	HAVE_D_NAMLEN
-		len = d->d_namlen;
-#else
-		len = 0;
-#endif
+		len = NLENGTH(d);
 	      }
 		
 	    if (fnmatch (pattern, name,
