@@ -1,19 +1,16 @@
-#include "config.h"
-#include "miscfn.h"
+#include "system.h"
 
 #ifdef HAVE_NETINET_IN_SYSTM_H
-# include <sys/types.h>
 # include <netinet/in_systm.h>
 #endif
 
 #include <netinet/in.h>
 #include <netinet/ip.h>
 #include <arpa/inet.h>
-#include <string.h>
 
-#include "inet_aton.h"
-
-int inet_aton(const char *cp, struct in_addr *inp) {
+int
+inet_aton(const char *cp, struct in_addr *inp)
+{
     long addr;
 
     addr = inet_addr(cp);
