@@ -1,6 +1,14 @@
+/* (C) 1998 Red Hat Software, Inc. -- Licensing details are in the COPYING
+   file accompanying popt source distributions, available from 
+   ftp://ftp.redhat.com/pub/code/popt */
+
 #ifndef H_POPT
 #define H_POPT
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #include <stdio.h>			/* for FILE * */
 
 #define POPT_OPTION_DEPTH	10
@@ -14,8 +22,18 @@
 					   set first in table; arg points 
 					   to callback, descrip points to 
 					   callback data to pass */
+#define POPT_ARG_INTL_DOMAIN    6       /* set the translation domain
+					   for this table and any
+					   included tables; arg points
+					   to the domain string */
+#define POPT_ARG_VAL		7	/* arg should take value val */
 #define POPT_ARG_MASK		0x0000FFFF
-#define POPT_ARGFLAG_MULTIPLE	0x80000000
+#define POPT_ARGFLAG_ONEDASH	0x80000000  /* allow -longoption */
+#define POPT_ARGFLAG_DOC_HIDDEN 0x40000000  /* don't show in help/usage */
+#define POPT_CBFLAG_PRE		0x80000000  /* call the callback before parse */
+#define POPT_CBFLAG_POST	0x40000000  /* call the callback after parse */
+#define POPT_CBFLAG_INC_DATA	0x20000000  /* use data from the include line,
+					       not the subtable */
 
 #define POPT_ERROR_NOARG	-10
 #define POPT_ERROR_BADOPT	-11
@@ -39,15 +57,15 @@ struct poptOption {
     int argInfo;
     void * arg;			/* depends on argInfo */
     int val;			/* 0 means don't return, just update flag */
-    char * descrip;		/* description for autohelp -- may be NULL */
-    char * argDescrip;		/* argument description for autohelp */
+    const char * descrip;	/* description for autohelp -- may be NULL */
+    const char * argDescrip;	/* argument description for autohelp */
 };
 
 struct poptAlias {
-    char * longName;		/* may be NULL */
+    const char * longName;	/* may be NULL */
     char shortName;		/* may be '\0' */
     int argc;
-    char ** argv;		/* must be free()able */
+    const char ** argv;		/* must be free()able */
 };
 
 extern struct poptOption poptHelpOptions[];
@@ -55,14 +73,19 @@ extern struct poptOption poptHelpOptions[];
 			0, "Help options", NULL },
 
 typedef struct poptContext_s * poptContext;
+#ifndef __cplusplus
 typedef struct poptOption * poptOption;
+#endif
 
-#define POPT_CB_USE_INCLUDE_DATA	((void *) -1)
+enum poptCallbackReason { POPT_CALLBACK_REASON_PRE, 
+			  POPT_CALLBACK_REASON_POST,
+			  POPT_CALLBACK_REASON_OPTION };
 typedef void (*poptCallbackType)(poptContext con, 
+				 enum poptCallbackReason reason,
 			         const struct poptOption * opt,
-				 const char * arg, void * data);
+				 const char * arg, const void * data);
 
-poptContext poptGetContext(char * name, int argc, char ** argv, 
+poptContext poptGetContext(const char * name, int argc, char ** argv, 
 			   const struct poptOption * options, int flags);
 void poptResetContext(poptContext con);
 
@@ -72,24 +95,30 @@ int poptGetNextOpt(poptContext con);
 char * poptGetOptArg(poptContext con);
 /* returns NULL if no more options are available */
 char * poptGetArg(poptContext con);
-char * poptPeekArg(poptContext con);
-char ** poptGetArgs(poptContext con);
+const char * poptPeekArg(poptContext con);
+const char ** poptGetArgs(poptContext con);
 /* returns the option which caused the most recent error */
-char * poptBadOption(poptContext con, int flags);
+const char * poptBadOption(poptContext con, int flags);
 void poptFreeContext(poptContext con);
-int poptStuffArgs(poptContext con, char ** argv);
+int poptStuffArgs(poptContext con, const char ** argv);
 int poptAddAlias(poptContext con, struct poptAlias alias, int flags);
-int poptReadConfigFile(poptContext con, char * fn);
+int poptReadConfigFile(poptContext con, const char * fn);
 /* like above, but reads /etc/popt and $HOME/.popt along with environment 
    vars */
 int poptReadDefaultConfig(poptContext con, int useEnv);
 /* argv should be freed -- this allows ', ", and \ quoting, but ' is treated
    the same as " and both may include \ quotes */
-int poptParseArgvString(char * s, int * argcPtr, char *** argvPtr);
+int poptParseArgvString(const char * s,
+		/*@out@*/ int * argcPtr, /*@out@*/ char *** argvPtr);
 const char * poptStrerror(const int error);
 void poptSetExecPath(poptContext con, const char * path, int allowAbsolute);
 void poptPrintHelp(poptContext con, FILE * f, int flags);
 void poptPrintUsage(poptContext con, FILE * f, int flags);
 void poptSetOtherOptionHelp(poptContext con, const char * text);
+const char * poptGetInvocationName(poptContext con);
+
+#ifdef  __cplusplus
+}
+#endif
 
 #endif
