@@ -1,3 +1,7 @@
+/* (C) 1998 Red Hat Software, Inc. -- Licensing details are in the COPYING
+   file accompanying popt source distributions, available from 
+   ftp://ftp.redhat.com/pub/code/popt */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -47,11 +51,11 @@ static void configLine(poptContext con, char * line) {
 	shortName = opt[1];
 
     if (!strcmp(entryType, "alias")) {
-	if (poptParseArgvString(line, &alias.argc, &alias.argv)) return;
+	if (poptParseArgvString(line, &alias.argc, (char ***)&alias.argv)) return;
 	alias.longName = longName, alias.shortName = shortName;
 	poptAddAlias(con, alias, 0);
     } else if (!strcmp(entryType, "exec")) {
-	con->execs = realloc(con->execs, 
+	con->execs = realloc(con->execs, /* XXX memory leak */
 				sizeof(*con->execs) * (con->numExecs + 1));
 	if (longName)
 	    con->execs[con->numExecs].longName = strdup(longName);
@@ -65,7 +69,7 @@ static void configLine(poptContext con, char * line) {
     }
 }
 
-int poptReadConfigFile(poptContext con, char * fn) {
+int poptReadConfigFile(poptContext con, const char * fn) {
     char * file, * chptr, * end;
     char * buf, * dst;
     int fd, rc;
@@ -83,7 +87,7 @@ int poptReadConfigFile(poptContext con, char * fn) {
     lseek(fd, 0, 0);
 
     file = alloca(fileLength + 1);
-    if ((fd = read(fd, file, fileLength)) != fileLength) {
+    if (read(fd, file, fileLength) != fileLength) {
 	rc = errno;
 	close(fd);
 	errno = rc;
