@@ -1,12 +1,10 @@
 #include "system.h"
 
-#include "build/rpmbuild.h"
+#include "rpmbuild.h"
 
 #include "build.h"
-#include "checksig.h"
 #include "install.h"
-#include "lib/signature.h"
-#include "popt/popt.h"
+#include "signature.h"
 
 #define GETOPT_REBUILD		1003
 #define GETOPT_RECOMPILE	1004
@@ -20,8 +18,7 @@
 #define GETOPT_SHOWRC		1018
 #define GETOPT_EXCLUDEPATH	1019
 #define	GETOPT_DEFINEMACRO	1020
-
-char * version = VERSION;
+#define	GETOPT_EVALMACRO	1021
 
 enum modes { MODE_QUERY, MODE_INSTALL, MODE_UNINSTALL, MODE_VERIFY,
 	     MODE_BUILD, MODE_REBUILD, MODE_CHECKSIG, MODE_RESIGN,
@@ -47,15 +44,13 @@ static int incldocs;
 static int initdb;
 static int justdb;
 static int noDeps;
-#ifdef	DELETE
-static int noFiles;
-#endif
 static int noGpg;
 static int noMd5;
 static int noOrder;
 static int noPgp;
 static int noScripts;
 static int noTriggers;
+static int noUsageMsg;
 static int oldPackage;
 static int showPercents;
 static char * pipeOutput;
@@ -70,7 +65,10 @@ static int showrc;
 static int signIt;
 static int test;
 
-static int rpm_version;
+static int showVersion;
+extern const char * rpmNAME;
+extern const char * rpmEVR;
+extern int rpmFLAGS;
 
 static struct rpmQVArguments rpmQVArgs;
 static struct rpmBuildArguments buildArgs;
@@ -86,8 +84,9 @@ static struct poptOption optionsTable[] = {
  { "build", 'b', POPT_ARG_STRING, 0, 'b',	NULL, NULL},
  { "checksig", 'K', 0, 0, 'K',			NULL, NULL},
  { "dbpath", '\0', POPT_ARG_STRING, 0, GETOPT_DBPATH,		NULL, NULL},
- { "define", '\0', POPT_ARG_STRING, 0, GETOPT_DEFINEMACRO,	NULL, NULL},
+ { "define", '\0', POPT_ARG_STRING, &noUsageMsg, GETOPT_DEFINEMACRO,NULL, NULL},
  { "erase", 'e', 0, 0, 'e',			NULL, NULL},
+ { "eval", '\0', POPT_ARG_STRING, &noUsageMsg, GETOPT_EVALMACRO, NULL, NULL},
  { "excludedocs", '\0', 0, &excldocs, 0,	NULL, NULL},
  { "excludepath", '\0', POPT_ARG_STRING, 0, GETOPT_EXCLUDEPATH,	NULL, NULL},
  { "force", '\0', 0, &force, 0,			NULL, NULL},
@@ -107,9 +106,6 @@ static struct poptOption optionsTable[] = {
  { "install", '\0', 0, 0, GETOPT_INSTALL,	NULL, NULL},
  { "justdb", '\0', 0, &justdb, 0,		NULL, NULL},
  { "nodeps", '\0', 0, &noDeps, 0,		NULL, NULL},
-#ifdef	DELETE
- { "nofiles", '\0', 0, &noFiles, 0,		NULL, NULL},
-#endif
  { "nogpg", '\0', 0, &noGpg, 0,			NULL, NULL},
  { "nomd5", '\0', 0, &noMd5, 0,			NULL, NULL},
  { "noorder", '\0', 0, &noOrder, 0,		NULL, NULL},
@@ -142,7 +138,7 @@ static struct poptOption optionsTable[] = {
  { "verbose", 'v', 0, 0, 'v',			NULL, NULL},
  { "verify", 'V', 0, 0, 'V',			NULL, NULL},
  {  NULL, 'y', 0, 0, 'V',			NULL, NULL},
- { "version", '\0', 0, &rpm_version, 0,		NULL, NULL},
+ { "version", '\0', 0, &showVersion, 0,		NULL, NULL},
  { NULL, '\0', POPT_ARG_INCLUDE_TABLE, 
 		rpmQVSourcePoptTable, 0,	(void *) &rpmQVArgs, NULL },
  { NULL, '\0', POPT_ARG_INCLUDE_TABLE, 
@@ -171,13 +167,12 @@ static void printUsage(void);
 static void printHelpLine(char * prefix, char * help);
 
 static void printVersion(void) {
-    fprintf(stdout, _("RPM version %s\n"), version);
+    fprintf(stdout, _("RPM version %s\n"), rpmEVR);
 }
 
 static void printBanner(void) {
     puts(_("Copyright (C) 1998 - Red Hat Software"));
-    puts(_("This may be freely redistributed under the terms of the GNU "
-	   "Public License"));
+    puts(_("This may be freely redistributed under the terms of the GNU GPL"));
 }
 
 static void printUsage(void) {
@@ -220,6 +215,7 @@ static void printUsage(void) {
     puts(_("                        [--nomd5] [targets]"));
     puts(_("       rpm {--setperms} [-afpg] [target]"));
     puts(_("       rpm {--setugids} [-afpg] [target]"));
+    puts(_("       rpm {--freshen -F} file1.rpm ... fileN.rpm"));
     puts(_("       rpm {--erase -e} [--root <dir>] [--noscripts] [--rcfile <file>]"));
     puts(_("                        [--dbpath <dir>] [--nodeps] [--allmatches]"));
     puts(_("                        [--justdb] [--notriggers] rpackage1 ... packageN"));
@@ -523,8 +519,8 @@ int main(int argc, char ** argv)
     const char * pkg;
     char * errString = NULL;
     poptContext optCon;
-    char * infoCommand[] = { "--info", NULL };
-    char * installCommand[] = { "--install", NULL };
+    const char * infoCommand[] = { "--info", NULL };
+    const char * installCommand[] = { "--install", NULL };
     int ec = 0;
     int status;
     int p[2];
@@ -534,6 +530,11 @@ int main(int argc, char ** argv)
     int upgrade = 0;
     int probFilter = 0;
 	
+#if HAVE_MCHECK_H && HAVE_MTRACE
+    mtrace();	/* Trace malloc only if MALLOC_TRACE=mtrace-output-file. */
+#endif
+    setprogname(argv[0]);	/* Retrofit glibc __progname */
+
     /* set the defaults for the various command line options */
     allFiles = 0;
     allMatches = 0;
@@ -553,15 +554,13 @@ int main(int argc, char ** argv)
     initdb = 0;
     justdb = 0;
     noDeps = 0;
-#ifdef	DELETE
-    noFiles = 0;
-#endif
     noGpg = 0;
     noMd5 = 0;
     noOrder = 0;
     noPgp = 0;
     noScripts = 0;
     noTriggers = 0;
+    noUsageMsg = 0;
     oldPackage = 0;
     showPercents = 0;
     pipeOutput = NULL;
@@ -574,7 +573,7 @@ int main(int argc, char ** argv)
     showrc = 0;
     signIt = 0;
     test = 0;
-    rpm_version = 0;
+    showVersion = 0;
 
     /* XXX Eliminate query linkage loop */
     parseSpecVec = parseSpec;
@@ -582,13 +581,13 @@ int main(int argc, char ** argv)
     specedit = NULL;
 
     /* set up the correct locale */
-    (void)setlocale(LC_ALL, "" );
+    setlocale(LC_ALL, "" );
 
 #ifdef	__LCLINT__
 #define	LOCALEDIR	"/usr/share/locale"
 #endif
-    (void)bindtextdomain(PACKAGE, LOCALEDIR);
-    (void)textdomain(PACKAGE);
+    bindtextdomain(PACKAGE, LOCALEDIR);
+    textdomain(PACKAGE);
 
     rpmSetVerbosity(RPMMESS_NORMAL);	/* XXX silly use by showrc */
 
@@ -757,63 +756,18 @@ int main(int argc, char ** argv)
 	    QVSource = RPMQV_ALL;
 	    break;
 
-#ifdef DYING	/* XXX WTFO? the above QVSources *must* be here */
-	  case GETOPT_WHATREQUIRES:
-	    if (QVSource != RPMQV_PACKAGE && 
-		QVSource != RPMQV_WHATREQUIRES)
-		argerror(_("one type of query/verify may be performed at a "
-				"time"));
-	    QVSource = RPMQV_WHATREQUIRES;
-	    break;
-
-	  case GETOPT_WHATPROVIDES:
-	    if (QVSource != RPMQV_PACKAGE && 
-		QVSource != RPMQV_WHATPROVIDES)
-		argerror(_("one type of query/verify may be performed at a "
-				"time"));
-	    QVSource = RPMQV_WHATPROVIDES;
-	    break;
-
-	  case GETOPT_TRIGGEREDBY:
-	    if (QVSource != RPMQV_PACKAGE && 
-		QVSource != RPMQV_TRIGGEREDBY)
-		argerror(_("one type of query/verify may be performed at a "
-				"time"));
-	    QVSource = RPMQV_TRIGGEREDBY;
-	    break;
-#endif	/* DYING */
-
 	  case GETOPT_REBUILD:
 	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_REBUILD)
 		argerror(_("only one major mode may be specified"));
 	    bigMode = MODE_REBUILD;
 	    break;
 
-#if XXX
-	  case GETOPT_RMSOURCE:
-	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_BUILD &&
-		bigMode != MODE_TARBUILD)
-		argerror(_("only one major mode may be specified"));
-	    rmsource = 1;
-	    break;
-#endif
-
 	  case GETOPT_RECOMPILE:
 	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_RECOMPILE)
 		argerror(_("only one major mode may be specified"));
 	    bigMode = MODE_RECOMPILE;
 	    break;
 
-#if XXX
-	  case GETOPT_BUILDROOT:
-	    if (bigMode != MODE_UNKNOWN &&
-		bigMode != MODE_BUILD && bigMode != MODE_REBUILD &&
-		bigMode != MODE_TARBUILD)
-		argerror(_("only one major mode may be specified"));
-	    buildRootOverride = optArg;
-	    break;
-#endif
-
 	  case GETOPT_RESIGN:
 	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_RESIGN)
 		argerror(_("only one major mode may be specified"));
@@ -841,6 +795,12 @@ int main(int argc, char ** argv)
 	    rpmDefineMacro(NULL, optArg, RMIL_CMDLINE);
 	    break;
 
+	  case GETOPT_EVALMACRO:
+	  { const char *val = rpmExpand(optArg, NULL);
+	    fprintf(stdout, "%s\n", val);
+	    xfree(val);
+	  } break;
+
 	  case GETOPT_TIMECHECK:
 	    tce = NULL;
 	    timeCheck = strtoul(optArg, &tce, 10);
@@ -865,7 +825,7 @@ int main(int argc, char ** argv)
 	    *errString++ = '\0';
 	    if (*errString != '/') 
 		argerror(_("relocations must have a / following the ="));
-	    relocations = realloc(relocations, 
+	    relocations = xrealloc(relocations, 
 				  sizeof(*relocations) * (numRelocations + 1));
 	    relocations[numRelocations].oldPath = optArg;
 	    relocations[numRelocations++].newPath = errString;
@@ -875,7 +835,7 @@ int main(int argc, char ** argv)
 	    if (*optArg != '/') 
 		argerror(_("exclude paths must begin with a /"));
 
-	    relocations = realloc(relocations, 
+	    relocations = xrealloc(relocations, 
 				  sizeof(*relocations) * (numRelocations + 1));
 	    relocations[numRelocations].oldPath = optArg;
 	    relocations[numRelocations++].newPath = NULL;
@@ -890,7 +850,7 @@ int main(int argc, char ** argv)
     if (quiet)
 	rpmSetVerbosity(RPMMESS_QUIET);
 
-    if (rpm_version) printVersion();
+    if (showVersion) printVersion();
     if (help) printHelp();
 
     if (arg < -1) {
@@ -1040,12 +1000,6 @@ int main(int argc, char ** argv)
 	argerror(_("--nodeps may only be specified during package "
 		   "building, installation, erasure, and verification"));
 
-#ifdef	DELETE
-    if (bigMode != MODE_VERIFY && noFiles)
-	argerror(_("--nofiles may only be specified during package "
-		   "verification"));
-#endif
-
     if (bigMode != MODE_INSTALL && bigMode != MODE_UNINSTALL &&
 	bigMode != MODE_BUILD && bigMode != MODE_TARBUILD && test )
 	argerror(_("--test may only be specified during package installation, "
@@ -1060,23 +1014,6 @@ int main(int argc, char ** argv)
     if (rootdir && rootdir[0] != '/')
 	argerror(_("arguments to --root (-r) must begin with a /"));
 
-#if XXX
-    if (bigMode != MODE_BUILD && bigMode != MODE_TARBUILD && clean)
-	argerror(_("--clean may only be used with -b and -t"));
-
-    if (bigMode != MODE_BUILD && bigMode != MODE_TARBUILD && rmsource)
-	argerror(_("--rmsource may only be used with -b and -t"));
-
-    if (bigMode != MODE_BUILD && bigMode != MODE_TARBUILD && buildArgs.shortCircuit) 
-	argerror(_("--short-circuit may only be used during package building"));
-
-    if (buildArgs.shortCircuit && (buildArgs.buildChar != 'c') && (buildArgs.buildChar != 'i')
-		     && (buildArgs.buildChar != 's')) {
-	argerror(_("--short-circuit may only be used with -bc, -bi, -bs, -tc "
-			"-ti, or -ts"));
-    }
-#endif
-
     if (oldPackage && !upgrade)
 	argerror(_("--oldpackage may only be used during upgrades"));
 
@@ -1122,7 +1059,10 @@ int main(int argc, char ** argv)
 	    struct stat sb;
 	    int errors = 0;
 
-	    argv = poptGetArgs(optCon);
+	    if ((argv = poptGetArgs(optCon)) == NULL) {
+		fprintf(stderr, _("no files to sign\n"));
+		errors++;
+	    } else
 	    while (*argv) {
 		if (stat(*argv, &sb)) {
 		    fprintf(stderr, _("cannot access file %s\n"), *argv);
@@ -1137,51 +1077,32 @@ int main(int argc, char ** argv)
 		switch (sigTag = rpmLookupSignatureType(RPMLOOKUPSIG_QUERY)) {
 		  case 0:
 		    break;
-		  case RPMSIGTAG_GPG:
 		  case RPMSIGTAG_PGP:
-		  case RPMSIGTAG_PGP5:
-		    if (sigTag == RPMSIGTAG_PGP
-		            && !rpmDetectPGPVersion(RPMSIGTAG_PGP)) {
+		    if ((sigTag == RPMSIGTAG_PGP || sigTag == RPMSIGTAG_PGP5) &&
+		        !rpmDetectPGPVersion(NULL)) {
 		        fprintf(stderr, _("pgp not found: "));
-			if (rpmDetectPGPVersion(RPMSIGTAG_PGP5)) {
-			    fprintf(stderr,
-  _("Use `%%_signature pgp5' instead of `%%_signature pgp' in macro file.\n"));
-			    exit(EXIT_FAILURE);
-		        }
-			/* Fall through to default: */
-		    }
-		    else if (sigTag == RPMSIGTAG_PGP5
-		    	    && !rpmDetectPGPVersion(RPMSIGTAG_PGP5)) {
-		        fprintf(stderr, _("pgp version 5 not found: "));
-			if (rpmDetectPGPVersion(RPMSIGTAG_PGP)) {
-			    fprintf(stderr,
-  _("Use `%%_signature pgp' instead of `%%_signature pgp5' in macro file.\n"));
-			    exit(EXIT_FAILURE);
-		        }
-			/* Fall through to default: */
-		    }
-		    else if (!(passPhrase = 
-			    rpmGetPassPhrase(_("Enter pass phrase: "), sigTag))) {
+			exit(EXIT_FAILURE);
+		    }	/* fall through */
+		  case RPMSIGTAG_GPG:
+		    passPhrase = rpmGetPassPhrase(_("Enter pass phrase: "), sigTag);
+		    if (passPhrase == NULL) {
 			fprintf(stderr, _("Pass phrase check failed\n"));
 			exit(EXIT_FAILURE);
-		    } else {
-			fprintf(stderr, _("Pass phrase is good.\n"));
-			passPhrase = strdup(passPhrase);
-			break;
 		    }
-		    /* Fall through */
+		    fprintf(stderr, _("Pass phrase is good.\n"));
+		    passPhrase = xstrdup(passPhrase);
+		    break;
 		  default:
 		    fprintf(stderr,
 		            _("Invalid %%_signature spec in macro file.\n"));
 		    exit(EXIT_FAILURE);
+		    break;
 		}
 	    }
 	} else {
 	    argerror(_("--sign may only be used during package building"));
 	}
-    }
-    else
-    {
+    } else {
     	/* Make rpmLookupSignatureType() return 0 ("none") from now on */
         rpmLookupSignatureType(RPMLOOKUPSIG_DISABLE);
     }
@@ -1204,7 +1125,7 @@ int main(int argc, char ** argv)
 	
     switch (bigMode) {
       case MODE_UNKNOWN:
-	if (!rpm_version && !help) printUsage();
+	if (!showVersion && !help && !noUsageMsg) printUsage();
 	break;
 
       case MODE_REBUILDDB:
@@ -1228,7 +1149,7 @@ int main(int argc, char ** argv)
 	if (!noPgp) checksigFlags |= CHECKSIG_PGP;
 	if (!noGpg) checksigFlags |= CHECKSIG_GPG;
 	if (!noMd5) checksigFlags |= CHECKSIG_MD5;
-	ec = doCheckSig(checksigFlags, poptGetArgs(optCon));
+	ec = rpmCheckSig(checksigFlags, (const char **)poptGetArgs(optCon));
 	/* XXX don't overflow single byte exit status */
 	if (ec > 255) ec = 255;
 	exit(ec);
@@ -1237,7 +1158,7 @@ int main(int argc, char ** argv)
       case MODE_RESIGN:
 	if (!poptPeekArg(optCon))
 	    argerror(_("no packages given for signing"));
-	ec = doReSign(addSign, passPhrase, poptGetArgs(optCon));
+	ec = rpmReSign(addSign, passPhrase, (const char **)poptGetArgs(optCon));
 	/* XXX don't overflow single byte exit status */
 	if (ec > 255) ec = 255;
 	exit(ec);
@@ -1261,7 +1182,7 @@ int main(int argc, char ** argv)
 	}
 
 	while ((pkg = poptGetArg(optCon))) {
-	    if (doSourceInstall("/", pkg, &specFile, &cookie))
+	    if (rpmInstallSource("/", pkg, &specFile, &cookie))
 		exit(EXIT_FAILURE);
 
 	    if (build(specFile, &buildArgs, passPhrase, 0, cookie, rcfile, force, noDeps)) {
@@ -1329,7 +1250,7 @@ int main(int argc, char ** argv)
 	if (noDeps) interfaceFlags |= UNINSTALL_NODEPS;
 	if (allMatches) interfaceFlags |= UNINSTALL_ALLMATCHES;
 
-	ec = doUninstall(rootdir, (const char **)poptGetArgs(optCon), 
+	ec = rpmErase(rootdir, (const char **)poptGetArgs(optCon), 
 			 uninstallFlags, interfaceFlags);
 	break;
 
@@ -1349,24 +1270,25 @@ int main(int argc, char ** argv)
 	if (ignoreOs) probFilter |= RPMPROB_FILTER_IGNOREOS;
 	if (ignoreSize) probFilter |= RPMPROB_FILTER_DISKSPACE;
 
-	if (allFiles) installFlags |= RPMTRANS_FLAG_ALLFILES;
-	if (justdb) installFlags |= RPMTRANS_FLAG_JUSTDB;
 	if (test) installFlags |= RPMTRANS_FLAG_TEST;
+	/* RPMTRANS_FLAG_BUILD_PROBS */
 	if (noScripts) installFlags |= RPMTRANS_FLAG_NOSCRIPTS;
+	if (justdb) installFlags |= RPMTRANS_FLAG_JUSTDB;
 	if (noTriggers) installFlags |= RPMTRANS_FLAG_NOTRIGGERS;
-
-	if (showPercents) interfaceFlags |= INSTALL_PERCENT;
-	if (showHash) interfaceFlags |= INSTALL_HASH;
-	if (noDeps) interfaceFlags |= INSTALL_NODEPS;
-	if (noOrder) interfaceFlags |= INSTALL_NOORDER;
-	if (upgrade) interfaceFlags |= INSTALL_UPGRADE;
-
 	if (!incldocs) {
 	    if (excldocs)
 		installFlags |= RPMTRANS_FLAG_NODOCS;
 	    else if (rpmExpandNumeric("%{_excludedocs}"))
 		installFlags |= RPMTRANS_FLAG_NODOCS;
 	}
+	if (allFiles) installFlags |= RPMTRANS_FLAG_ALLFILES;
+	/* RPMTRANS_FLAG_KEEPOBSOLETE */
+
+	if (showPercents) interfaceFlags |= INSTALL_PERCENT;
+	if (showHash) interfaceFlags |= INSTALL_HASH;
+	if (noDeps) interfaceFlags |= INSTALL_NODEPS;
+	if (noOrder) interfaceFlags |= INSTALL_NOORDER;
+	if (upgrade) interfaceFlags |= INSTALL_UPGRADE;
 
 	if (!poptPeekArg(optCon))
 	    argerror(_("no packages given for install"));
@@ -1378,13 +1300,13 @@ int main(int argc, char ** argv)
 	    relocations[0].newPath = prefix;
 	    relocations[1].oldPath = relocations[1].newPath = NULL;
 	} else if (relocations) {
-	    relocations = realloc(relocations, 
+	    relocations = xrealloc(relocations, 
 				  sizeof(*relocations) * (numRelocations + 1));
 	    relocations[numRelocations].oldPath = NULL;
 	    relocations[numRelocations].newPath = NULL;
 	}
 
-	ec += doInstall(rootdir, (const char **)poptGetArgs(optCon), 
+	ec += rpmInstall(rootdir, (const char **)poptGetArgs(optCon), 
 			installFlags, interfaceFlags, probFilter, relocations);
 	break;
 
@@ -1405,11 +1327,7 @@ int main(int argc, char ** argv)
 
       case MODE_VERIFY:
 	verifyFlags = (VERIFY_FILES|VERIFY_DEPS|VERIFY_SCRIPT|VERIFY_MD5);
-#ifdef	DELETE
-	if (!noFiles) verifyFlags |= VERIFY_FILES;
-#else
 	verifyFlags &= ~qva->qva_flags;
-#endif
 	if (noDeps)	verifyFlags &= ~VERIFY_DEPS;
 	if (noScripts)	verifyFlags &= ~VERIFY_SCRIPT;
 	if (noMd5)	verifyFlags &= ~VERIFY_MD5;
@@ -1430,6 +1348,8 @@ int main(int argc, char ** argv)
     }
 
     poptFreeContext(optCon);
+    freeMacros(NULL);
+    rpmFreeRpmrc();
 
     if (pipeChild) {
 	fclose(stdout);
