@@ -33,9 +33,7 @@
 /* <unistd.h> should be included before any preprocessor test
    of _POSIX_VERSION.  */
 #ifdef HAVE_UNISTD_H
-/*@-skipposixheaders@*/
 #include <unistd.h>
-/*@=skipposixheaders@*/
 #endif
 
 #if TIME_WITH_SYS_TIME
@@ -186,12 +184,49 @@ char *alloca ();
 #include <limits.h>
 #endif
 
+#if HAVE_ERR_H
+#include <err.h>
+#endif
+
 #if HAVE_MALLOC_H
 #include <malloc.h>
 #endif
 
+/*@only@*/ void * xmalloc (size_t size);
+/*@only@*/ void * xcalloc (size_t nmemb, size_t size);
+/*@only@*/ void * xrealloc (void *ptr, size_t size);
+/*@only@*/ char * xstrdup (const char *str);
+void *vmefail(void);
+
+#if HAVE_MCHECK_H
+#include <mcheck.h>
+#endif
+
+
+/* Memory allocation via macro defs to get meaningful locations from mtrace() */
+#if HAVE_MCHECK_H && defined(__GNUC__)
+#define	xmalloc(_size) 		(malloc(_size) ? : vmefail())
+#define	xcalloc(_nmemb, _size)	(calloc((_nmemb), (_size)) ? : vmefail())
+#define	xrealloc(_ptr, _size)	(realloc((_ptr), (_size)) ? : vmefail())
+#define	xstrdup(_str)	(strcpy((malloc(strlen(_str)+1) ? : vmefail()), (_str)))
+#endif	/* HAVE_MCHECK_H && defined(__GNUC__) */
+
+/* Retrofit glibc __progname */
+#if defined __GLIBC__ && __GLIBC__ >= 2
+#define	setprogname(pn)
+#else
+#define	__progname	program_name
+#define	setprogname(pn)	\
+  { if ((__progname = strrchr(pn, '/')) != NULL) __progname++; \
+    else __progname = pn;		\
+  }
+#endif
+char *__progname;
+
 #if HAVE_NETDB_H
+#ifndef __LCLINT__
 #include <netdb.h>
+#endif	/* __LCLINT__ */
 #endif
 
 #if HAVE_PWD_H
@@ -216,6 +251,8 @@ char *alloca ();
 # undef textdomain
 # define textdomain(Domain) /* empty */
 # define _(Text) Text
+# undef dgettext
+# define dgettext(DomainName, Text) Text
 #endif
 
 #define N_(Text) Text
@@ -261,13 +298,20 @@ extern void *myrealloc(void *, size_t);
 
 #if HAVE_SYS_SOCKET_H
 #include <sys/types.h>
+#ifndef	__LCLINT__
 #include <sys/socket.h>
+#endif	/* __LCLINT__ */
 #endif
 
 #if HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
 
+/* Solaris <= 2.6 limits getpass return to only 8 chars */
+#if HAVE_GETPASSPHRASE
+#define	getpass	getpassphrase
+#endif
+
 #if ! HAVE_LCHOWN
 #define lchown chown
 #endif
