@@ -32,11 +32,11 @@ static void dpf(char *format, ...)
 #define	DPRINTF(_lvl, _fmt)
 #endif
 
-const char *program_name = NULL;
 int debug = MYDEBUG;
 int verbose = 0;
 char *inputdir = NULL;
 char *outputdir = NULL;
+int msgid_too = 0;
 int gottalang = 0;
 int nlangs = 0;
 char *onlylang[128];
@@ -74,7 +74,7 @@ getTagVal(const char *tname)
     return tval;
 }
 
-const struct headerTypeTableEntry {
+static const struct headerTypeTableEntry {
     char *name;
     int val;
 } rpmTypeTable[] = {
@@ -122,11 +122,11 @@ genSrpmFileName(Header h)
 
 #if 0
     if (strcmp(sourcerpm, sfn))
-	return strdup(sourcerpm);
+	return xstrdup(sourcerpm);
 
     return NULL;
 #else
-    return strdup(sourcerpm);
+    return xstrdup(sourcerpm);
 #endif
 
 }
@@ -153,10 +153,6 @@ hasLang(const char *onlylang, char **langs, char **s)
 /* ================================================================== */
 /* XXX stripped down gettext environment */
 
-#define	xstrdup		strdup
-#define	xmalloc		malloc
-#define	xrealloc	realloc
-
 #if !defined(PARAMS)
 #define	PARAMS(_x)	_x
 #endif
@@ -711,10 +707,16 @@ headerInject(Header h, int *poTags, message_list_ty *mlp)
 	}
 
 	e = *s;
+
 	/* Search for the msgid ... */
 	if ((mp = message_list_search(mlp, e)) == NULL)
 		goto bottom;
-DPRINTF(1, ("%s\n\tmsgid\n", getTagString(*tp)));
+DPRINTF(1, ("%s\n\tmsgid", getTagString(*tp)));
+	if (msgid_too) {
+DPRINTF(1, (" (drilled)"));
+	    headerAddI18NString(h, *tp, e, "C");
+	}
+DPRINTF(1, ("\n"));
 
 	/* Skip fuzzy ... */
 	if (mp->is_fuzzy) {
@@ -998,9 +1000,9 @@ main(int argc, char **argv)
     int errflg = 0;
     FD_t fdi;
 
-    program_name = basename(argv[0]);
+    setprogname(argv[0]);	/* Retrofit glibc __progname */
 
-    while((c = getopt(argc, argv, "degEMl:C:I:O:Tv")) != EOF)
+    while((c = getopt(argc, argv, "defgEMl:C:I:O:Tv")) != EOF)
     switch (c) {
     case 'C':
 	mastercatalogue = strdup(optarg);
@@ -1014,6 +1016,9 @@ main(int argc, char **argv)
     case 'E':
 	message_print_style_escape(1);
 	break;
+    case 'f':
+	msgid_too++;
+	break;
     case 'l':
 	gottalang = 1;
 	onlylang[nlangs++] = strdup(optarg);
@@ -1051,7 +1056,7 @@ main(int argc, char **argv)
 
     fdi = fdDup(STDIN_FILENO);
 
-    if (!strcmp(program_name, RPMGETTEXT)) {
+    if (!strcmp(__progname, RPMGETTEXT)) {
 	if (optind == argc) {
 	    rc = rpmgettext(fdi, STDINFN, stdout);
 	} else {
@@ -1060,15 +1065,15 @@ main(int argc, char **argv)
 		    break;
 	    }
 	}
-    } else if (!strcmp(program_name, RPMPUTTEXT)) {
+    } else if (!strcmp(__progname, RPMPUTTEXT)) {
 	if (mastercatalogue == NULL) {
 		fprintf(stderr, _("%s: must specify master PO catalogue with -C\n"),
-			program_name);
+			__progname);
 		exit(1);
         }
 	if (optind == argc) {
 		fprintf(stderr, _("%s: no binary rpms on cmd line\n"),
-			program_name);
+			__progname);
 		exit(1);
 	} else {
 	    string_list_ty *drillp = string_list_alloc();
@@ -1078,7 +1083,7 @@ main(int argc, char **argv)
 	    rc = rpmputtext(fdi, mastercatalogue, stdout, drillp);
 	    string_list_free(drillp);
 	}
-    } else if (!strcmp(program_name, RPMCHKTEXT)) {
+    } else if (!strcmp(__progname, RPMCHKTEXT)) {
 	if (optind == argc) {
 	    rc = rpmchktext(fdi, STDINFN, stdout);
 	} else {
