@@ -11,7 +11,9 @@ int main(int argc, char **argv)
     FD_t fdi, fdo;
     struct rpmlead lead;
     Header hd;
+    int rc;
     
+    setprogname(argv[0]);	/* Retrofit glibc __progname */
     if (argc == 1) {
 	fdi = fdDup(STDIN_FILENO);
     } else {
@@ -27,7 +29,7 @@ int main(int argc, char **argv)
     hd = headerRead(fdi, (lead.major >= 3) ?
 		    HEADER_MAGIC_YES : HEADER_MAGIC_NO);
     fdo = fdDup(STDOUT_FILENO);
-    headerWrite(fdo, hd, HEADER_MAGIC_YES);
+    rc = headerWrite(fdo, hd, HEADER_MAGIC_YES);
     
-    return 0;
+    return rc;
 }
